﻿function BaseView() {
    this._inheritFrom = MoveableElement;
    this._inheritFrom();

    this._guid = 0;
    this._world = null;
    this._evX;
    this._evY;
    this._lastInputX = 0;
    this._lastInputY = 0;
    this._inputDown = false;
    this._mouseTicksDown = 0;
    this._inputActioned = false;
    this._elements = new Array();
    this._children = new Array();
    this._isVisible = true;
    this._parent = null;

    this._id = "";
    this._x = 0;
    this._y = 0;
    this._layerCount = 1;

    this._opacity = 1;
    this._fadeDirection = 0;

    this._dispose = function () {
        if (this._world != null) {
            this._world._dispose();
            this._world = null;
        }

        this._elements = null;

        for (var i = 0; i < this._children.length; i++) {
            this._children[i]._dispose();
        }

        this._children = null;
        this._parent = null;
    }

    this._init = function (cb) {
        this._load(cb);
    }

    this._onResize = function() {
        if (this._world != null) {
            this._world._width = _backSurface1.width;
            this._world._height = _backSurface1.height;

            if (_modelManager != null) {
                this._setScale(true);
            }

            return true;
        }

        return false;
    }

    this._setScale = function (_hasResized) {
        var sx = 420;
        var sy = 420;

        /*if (this._world._width > this._world._height) {
            sx = 680;
            sy = 520;
        }*/

        _screenScale = this._world._width / sx;
        var _vscale = this._world._height / sy;
        if (_vscale < _screenScale) {
            _screenScale = _vscale;
        }

        //_screenScale = 1;

        if (_screenScale > 4) {
            _screenScale = 4;
        }

        _screenScale2 = 1 / _screenScale;

        if (_hasResized) {
            this._resized(_screenScale, _screenScale);
        }

        return false;
    }

    this._resized = function(sx, sy) {
        if (this._world != null) {
            _modelManager._applyScale(sx, sy);
            this._resizeWorld(sx, sy);
        }
    }

    this._resizeWorld = function (sx, sy) {
        this._world._width = _backSurface1.width;
        this._world._height = _backSurface1.height;

        this._updateCameras();
        this._world._updateViewingScale(sx, sy);

        for (var i = 0; i < this._children.length; i++) {
            this._children[i]._resizeWorld(sx, sy);
        }
    }

    this._load = function (cb) {
        console.log("View Load");

        if (this._id == "") {
            console.log("View ID is Empty");
            return false;
        }

        var th = this;
        var s = $(_mainXml).find('screens scr[id="' + this._id + '"]');
        var img = new Object();
        var mds = new Array();
        var c = 0;

        c += this._findAssets(s, mds, img);

        if (c > 0) {
            _imageManager._loadBatch(img, function () {
                th._assetsLoaded(s, mds, cb);
            });
        }
        else {
            this._assetsLoaded(s, mds, cb);
        }

        return true;
    }

    this._findAssets = function (s, mds, img) {
        var c = 0;
        var th = this;

        s.find('o,p').each(function () {
            var mo = $(this).attr('m');

            if (!_modelManager._doesModelExist(mo)) {
                var m = $(_mainXml).find('models m[id="' + mo + '"]');
                mds.push(m);
                m.find('p[s^="img/"]').each(function () {
                    var n = $(this).attr('s');
                    var f = $(this).attr('f');
                    if (f == null) {
                        f = "png";
                    }

                    img[n] = n + "." + f;
                    c++;
                });
            }
        });

        s.find('s').each(function () {
            var id = $(this).attr('m');
            console.log("Found child screen: " + id);

            var s = $(_mainXml).find('screens scr[id="' + id + '"]');
            th._findAssets(s, mds, img);
        });

        return c;
    }

    this._assetsLoaded = function (s, mds, cb) {
        console.log("Assets Loaded");

        this._initWorld(this._layerCount);

        if (mds) {
            for (var i = 0; i < mds.length; i++) {
                _modelManager._parseModel(mds[i], this._world._camera);
            }
        }

        this._setup(s, cb);
        this._initChildren(s);
    }

    this._initChildren = function (s) {
        var th = this;

        s.find('s').each(function () {
            var id = $(this).attr('m');
            var x = $(this).attr('x');
            var y = $(this).attr('y');
            var cs = $(_mainXml).find('screens scr[id="' + id + '"]');
            var t = cs.attr('t');
            var o = eval("new " + t + "()");

            ApplyAttributes(o, $(this));
            o._id = id;

            if (x) {
                o._x = x >> 0;
            }
            if (y) {
                o._y = y >> 0;
            }
            o._assetsLoaded(cs, null, null)
            o._parent = th;

            th._children.push(o);
            th._elements[$(this).attr('id')] = o;
        });
    }

    this._initWorld = function (l) {
        if (this._world != null) {
            this._world._dispose();
        }

        this._world = new GameWorld(l);
        this._world._width = _backSurface1.width;
        this._world._height = _backSurface1.height;
        this._world._camera._viewport._width = _backSurface1.width;
        this._world._camera._viewport._height = _backSurface1.height;

        this._setScale(false);
        this._createCameras();

        this._updateCameras();
        this._updateOtherCameras();
    }

    this._setup = function (s, cb) {
        var th = this;

        s.find("o").each(function () {
            var mo = $(this).attr('m');
            var id = $(this).attr('id') >> 0;
            var x = $(this).attr('x') >> 0;
            var y = $(this).attr('y') >> 0;
            var l = $(this).attr('l') >> 0;
            var a = "";
            var r = 0;

            if ($(this).attr('r') != null) {
                r = $(this).attr('r') * 1;
            }
            if ($(this).attr('a') != null) {
                a = $(this).attr('a') + "";
            }

            var m = th._getModelInstance(mo, x, y);

            ApplyAttributes(m, $(this));

            if (a != "") {
                m._play(a);
            }

            m._setRotate(r);
            m._sourceTransformChanged(_screenScale, _screenScale);

            th._world._addObject(l, m);
            if($(this).attr('id') != null) {
                m._id = $(this).attr('id') + '';
                th._elements[m._id] = m;
            }
        });

        this._updateOtherCameras();
        if (cb) {
            cb(this);
        }

        this._start();
    }

    this._start = function () {
    }

    this._getModelInstance = function (mo, x, y) {
        var m = null;

        if (_modelManager._doesModelExist(mo)) {
            m = _modelManager._addModelFromCache(mo, x, y);
        }
        else {
            console.log("Model missing:" + mo);
        }

        return m;
    }

    this._createCameras = function () {

    }

    this._updateCameras = function () {
        var hw = (this._world._width >> 1) * _screenScale2;
        var hh = (this._world._height >> 1) * _screenScale2;
        this._world._camera._x = -hw;
        this._world._camera._y = -hh;
    }

    this._updateOtherCameras = function () {
    }

    this._update = function (t, x, y) {
        if (this._isVisible) {
            x += this._x;
            y += this._y;

            if (this._world) {
                this._renderAndUpdate(t, x, y);
            }

            for (var i = 0; i < this._children.length; i++) {
                this._children[i]._update(t, x, y);
            }
        }
    }

    this._renderAndUpdate = function (t, x, y) {
    }

    this._inputDownProcess = function (x, y) {
        if (this._isVisible) {
            this._lastInputX = x;
            this._lastInputY = y;

            if (!this._enableDraggingControl || this._isGameOver || this._isPaused || _mainPopupVisible || this._isBusy || this._isProcessing || this._timingOut) {
                return;
            }

            this._cPos(x, y);

            this._inputActioned = false;
        }
        return false;
    }

    this._inputMoveProcess = function (x, y) {
        if (this._isVisible) {
            if (!this._enableDraggingControl || this._isGameOver || this._isPaused || this._timingOut || _mainPopupVisible) {
                return;
            }

            if (this._inputDown && this._selected != null) {
                this._cPos(x, y);

                var diffX = ((x - this._lastInputX) * _screenScale2) * _pixelRatio;
                var diffY = ((y - this._lastInputY) * _screenScale2) * _pixelRatio;

                this._lastInputX = x;
                this._lastInputY = y;
            }
        }

        return false;
    }

    this._inputUpProcess = function () {
        if (this._isVisible) {
            if (this._world == null || this._demo || this._isGameOver || this._isPaused || this._timingOut) {
                return;
            }

            if (this._inputDown) {
            }
        }
    }

    this._cPos = function (x, y) {
        var o = $(_backSurface1).offset();
        x -= o.left;
        y -= o.top;

        this._evX = this._world._camera._x + (x * _screenScale2 * _pixelRatio);
        this._evY = this._world._camera._y + (y * _screenScale2 * _pixelRatio);
    }

    this._fadeOut = function () {
        this._fadeDirection = -0.025;
    }

    this._updateFade = function () {
        this._opacity += this._fadeDirection;
        if (this._opacity <= 0) {
            this._opacity = 0;
            return false;
        }
        else if (this._opacity >= 1) {
            this._opacity = 1;
            return false;
        }

        return true;
    }
}