﻿/* Image Manager */
function ImageManager() {
    this._baseURL = "";
    this._images = {};
    this._toLoadCount = 0;
    this._loadedCount = 0;
    this._loaded = false;
    this._onLoaded = null;

    this._load = function (list) {
        var img;

        for (x in list) {
            this._toLoadCount++;

            img = new Image();
            img.onload = this._loadingComplete;
            img.src = this._baseURL + list[x];
            img.im = this;

            this._images[x] = img;
        }
    }

    this._loadingComplete = function (e) {
        var im = this.im;
        im._loadedCount++;

        if (im._loadedCount == im._toLoadCount) {
            im._loaded = true;
            if (im._onLoaded != null) {
                im._onLoaded();
            }
        }
        this.im = null;
    }

    this._loadBatch = function (list, cb) {
        var img;
        var ct = 0;

        for (x in list) {
            ct++;
        }

        _mainImageLoadCount = ct;
        _mainImageCountRemaining = ct;

        console.log("Loading:" + _mainImageLoadCount + " - " + _mainImageCountRemaining);

        for (x in list) {
            if (this._images[x] == null) {
                img = new Image();
                img.onload = function () {
                    ct--;
                    if (ct == 0) {
                        cb();
                    }
                    _mainImageCountRemaining = ct;
                };

                var u = this._baseURL + list[x];

                if (list[x] == "img/sgl.png") {
                    u = _vlxLogo;
                }

                img.src = u;

                this._images[x] = img;
            }
            else {
                ct--;
                if (ct == 0) {
                    cb();
                }
                _mainImageCountRemaining = ct;
            }
        }
    }
}

function GameWorld(lc) {
    this._layers = new Array();
    this._background = "#000000";
    this._width = 0;
    this._height = 0;
    this._camera = new Camera();
    this._layerRenderComplete = null;
    this._objectCountRendered = 0;

    this._getPosXFromScreenCoord = function (x) {
        var v = 1 / _screenScale;
        x = (x * v);
        return x;
    }

    this._getPosYFromScreenCoord = function (y) {
        var v = 1 / _screenScale;
        y = (y * v);
        return y;
    }

    this._dispose = function () {
        var i;

        for (i = 0; i < this._layers._count; i++) {
            var l = this._layers[i];
            for (var a = 0; a < l.length; a++) {
                l[a]._dispose();
            }
            this._layers[i] = null;
        }

        this._camera._dispose();

        this._camera = null;
        this._layers = null;
    }

    this._initLayers = function (lc) {
        var i;

        for (i = 0; i < lc; i++) {
            this._layers.push(new Array());
            this._layers[i]._clipRect = null;
            this._layers[i]._clipList = null;
            this._layers[i]._camera = this._camera;
        }
    }
    this._initLayers(lc);

    this._changezIndex = function (obj, l, z) {
        var i = this._indexOfObjectInLayer(obj, this._layers[l]);

        if (i >= 0) {
            obj._zIndex = z;
            this._layers[l].splice(i, 1);
            this._addObject(l, obj);
        }
    }

    this._indexOfObjectInLayer = function (obj, l) {
        var i;
        for (i = 0; i < l.length; i++) {
            if (l[i] == obj) {
                return i;
            }
        }

        return -1;
    }

    this._addObject = function (l, obj) {
        var i = 0;

        for (i = 0; i < this._layers[l].length; i++) {
            if (obj._zIndex >= this._layers[l][i]._zIndex) {
                this._layers[l].splice(i, 0, obj);
                obj._addedToWorld();
                return;
            }
        }
        this._layers[l].push(obj);
        obj._addedToWorld();
    }

    this._addObjectAtStart = function (l, obj) {
        this._layers[l].unshift(obj);
        obj._addedToWorld();
    }

    this._remove = function (l, obj) {
    }

    this._renderAndUpdate = function (context) {
        var i;
        var a;

        this._objectCountRendered = 0;
        this._clearBackground(context);

        for (i = 0; i < this._layers.length; i++) {
            var l = this._layers[i];

            this._renderAndUpdateLayer(context, l, 0, 0);
        }
    }

    this._renderAndUpdateLayer = function (context, l, x, y) {
        context.save();
        if (l._clipList != null) {
            for (a = l._clipList.length - 1; a >= 0; a--) {
                var e = l._clipList[a];
                e._render(context, l._camera, x, y);

                if (!e._update()) {
                    var i = this._indexOfObjectInLayer(e, l);

                    if (i >= 0) {
                        e._dispose();
                        l.splice(i, 1);
                    }
                }
                this._objectCountRendered++;
            }
        }
        else {
            if (l._clipRect != null) {
                context.beginPath();
                for (var a = 0; a < l._clipRect.length; a++) {
                    context.rect(l._clipRect[a]._x, l._clipRect[a]._y, l._clipRect[a]._width, l._clipRect[a]._height);
                }
                context.clip();
            }

            for (a = l.length - 1; a >= 0; a--) {
                l[a]._render(context, l._camera, x, y);

                if (!l[a]._update()) {
                    l[a]._dispose();
                    l.splice(a, 1);
                }
                this._objectCountRendered++;
            }
        }
        context.restore();
        if (this._layerRenderComplete != null) {
            this._layerRenderComplete(this, i);
        }
    }

    this._renderLayer = function (context, l, x, y) {
        context.save();
        if (l._clipList != null) {
            for (a = l._clipList.length - 1; a >= 0; a--) {
                var e = l._clipList[a];
                e._render(context, l._camera, x, y);
                this._objectCountRendered++;
            }
        }
        else {
            if (l._clipRect != null) {
                context.beginPath();
                for (var a = 0; a < l._clipRect.length; a++) {
                    context.rect(l._clipRect[a]._x, l._clipRect[a]._y, l._clipRect[a]._width, l._clipRect[a]._height);
                }
                context.clip();
            }

            for (a = l.length - 1; a >= 0; a--) {
                l[a]._render(context, l._camera, x, y);
                this._objectCountRendered++;
            }
        }
        context.restore();
        if (this._layerRenderComplete != null) {
            this._layerRenderComplete(this, i);
        }
    }

    this._updateLayer = function (context, l, x, y) {
        if (l._clipList != null) {
            for (a = l._clipList.length - 1; a >= 0; a--) {
                var e = l._clipList[a];

                if (!e._update()) {
                    var i = this._indexOfObjectInLayer(e, l);

                    if (i >= 0) {
                        e._dispose();
                        l.splice(i, 1);
                    }
                }
            }
        }
        else {
            for (a = l.length - 1; a >= 0; a--) {
                if (!l[a]._update()) {
                    l[a]._dispose();
                    l.splice(a, 1);
                }
            }
        }
    }

    this._renderAndUpdateLayerRange = function (context, l, cl, sx, ex, sdx) {
        context.save();

        cl._hitList = new Array();
        if (l.length == 0) {
            return;
        }

        if (sdx >= 0) {
            for (a = cl._start; a >= 0; a--) {
                var o = l[a];

                if (o._x + o._bidDim >= sx) {
                    if (o._x - o._bidDim < ex) {
                        if (o._isVisible) {
                            cl._hitList.push(a);
                            o._render(context, l._camera, 0, 0);

                            if (!o._update()) {
                                o._dispose();
                                l.splice(a, 1);
                            }
                            this._objectCountRendered++;
                        }
                    }
                    else {
                        cl._end = a;
                        break;
                    }
                }
                else {
                    cl._start--;
                }
            }
        }
        else {
            for (a = cl._end; a < l.length; a++) {
                var o = l[a];

                if (o._x - o._bidDim <= ex) {
                    if (o._x + o._bidDim > sx) {
                        if (o._isVisible) {
                            cl._hitList.push(a);
                            o._render(context, l._camera, 0, 0);

                            if (!o._update()) {
                                o._dispose();
                                l.splice(a, 1);
                            }
                            this._objectCountRendered++;
                        }
                    }
                    else {
                        cl._start = a;
                        break;
                    }
                }
                else {
                    cl._end++;
                }
            }
        }

        if (cl._end >= l.length) {
            cl._end = l.length - 1;
        }
        context.restore();
    }

    this._renderLayerRange = function (context, l, cl, sx, ex, sdx, sy, ey) {
        context.save();

        cl._hitList = new Array();
        if (l.length == 0) {
            return;
        }

        for (a = cl._end; a < l.length; a++) {
            var o = l[a];
            if (o._y - o._bidDim <= ey) {
                if (o._y + o._bidDim > sy) {
                    cl._hitList.push(a);
                }
                else {
                    cl._start = a;
                    break;
                }
            }
            else {
                cl._end++;
            }
        }

        for (a = cl._hitList.length - 1; a >= 0; a--) {
            var o = l[cl._hitList[a]];
            o._render(context, l._camera, 0, 0);
        }

        if (cl._end >= l.length) {
            cl._end = l.length - 1;
        }
        context.restore();
    }

    this._updateLayerRange = function (context, l, cl, sx, ex, sdx, sy, ey) {
        cl._hitList = new Array();
        if (l.length == 0) {
            return;
        }

        for (a = cl._end; a < l.length; a++) {
            var o = l[a];
            if (o._y - o._bidDim <= ey) {
                if (o._y + o._bidDim > sy) {
                    if (o._isVisible) {
                        cl._hitList.push(a);

                        if (!o._update()) {
                            o._dispose();
                            l.splice(a, 1);
                        }
                    }
                }
                else {
                    cl._start = a;
                    break;
                }
            }
            else {
                cl._end++;
            }
        }

        if (cl._end >= l.length) {
            cl._end = l.length - 1;
        }
    }

    this._fillBackground = function (context, f) {
        context.fillStyle = f;
        context.fillRect(0, 0, this._width, this._height);
    }

    this._clearBackground = function (context) {
        context.fillStyle = this._background;
        context.clearRect(0, 0, this._width, this._height);
    }

    this._updateViewingScale = function (sx, sy) {
        var i;

        for (i = 0; i < this._layers.length; i++) {
            var l = this._layers[i];
            for (var a = 0; a < l.length; a++) {
                var o = l[a];
                for (var b = 0; b < o._children.length; b++) {
                    var p = o._children[b];
                    p._setSourceTransforms(sx, sy);
                }
                o._sourceTransformChanged(sx, sy);
            }
        }
    }

    this._drawPoint = function (context, x, y) {
        context.save();

        context.drawImage(_imageManager._images["img/d"], x - 0, y - 0);
        context.restore();
    }
}

function MoveableElement() {
    this._targetX = 0;
    this._targetY = 0;
    this._targetRotate = 0;
    this._targetMoveScaler = 0;
    this._removeWhenAtTarget = false;
    this._useTarget = false;
    this._atTargetCallback = null;

    this._reset = function () {
        this._useTarget = false;
    }

    this._moveTo = function (tx, ty, scale, removeWhenAtTarget, cb) {
        this._targetX = tx;
        this._targetY = ty;
        this._targetMoveScaler = scale;
        this._useTarget = true;
        this._removeWhenAtTarget = removeWhenAtTarget;
        this._atTargetCallback = cb;
    }

    this._updateMoveable = function () {
        if (this._useTarget) {
            var diffX = (this._targetX - this._x) * this._targetMoveScaler;
            var diffY = (this._targetY - this._y) * this._targetMoveScaler;

            this._x += diffX;
            this._y += diffY;

            if (this._x >= this._targetX - 1 && this._x <= this._targetX + 1 && this._y >= this._targetY - 1 && this._y <= this._targetY + 1) {
                this._useTarget = false;
                this._x = this._targetX;
                this._y = this._targetY;
                if (this._removeWhenAtTarget) {
                    return false;
                }
                else if (this._atTargetCallback) {
                    this._atTargetCallback(this);
                }
            }
        }
        return true;
    }
}

/* RenderPrimitive */
function BaseElement() {
    this._id = "";
    this._x = 0;
    this._y = 0;
    this._width = 0;
    this._height = 0;
    this._halfWidth = 0;
    this._halfHeight = 0;
    this._scaleX = 1;
    this._scaleY = 1;
    this._rotate = 0;
    this._rotateRadians = 0;
    this._rotateRadiansCos = 0;
    this._rotateRadiansSin = 0;
    this._transformedWidth = 0;
    this._transformedHeight = 0;
    this._transformedHalfWidth = 0;
    this._transformedHalfHeight = 0;
    this._isVisible = true;
    this._direction = new Point2(0, 0);
    this._speed = 0;
    this._opacity = 1;
    this._isMoveable = false;
    this._boundingBox = null;
    this._clipAction = 0;
    this._initDelay = 0;
    this._clipRect = null;
    this._isFixed = false;

    this._onRenderComplete = null;
    this._onRemove = null;
    this._onPreMove = null;
    this._onMoved = null;
    this._onUpdated = null;

    this._alphaFadeout = -1;
    this._alphaFadeoutDelay = -1;
    this._alphaFadeoutStep = 0;
    this._alphaFadein = -1;
    this._alphaFadeinDelay = -1;
    this._alphaFadeinStep = 0;
    this._removeOnFadeout = false;
    this._onFadeoutComplete = null;

    this._clone = function () {
        return new BaseElement();
    }

    this._dispose = function () {
        this._onRenderComplete = null;
        this._onRemove = null;
        this._onPreMove = null;
        this._onMoved = null;
        this._onUpdated = null;
        this._onFadeoutComplete = null;
    }

    this._setPosition = function (x, y) {
        this._x = x;
        this._y = y;
    }

    this._setX = function (value) {
        this._x = value;
    }

    this._setY = function (value) {
        this._y = value;
    }

    this._setOpacity = function (value) {
        this._opacity = value;
    }

    this._setRotate = function (r) {
        this._rotate = r;
        this._rotateRadians = MatrixUtility_DegToRad(r);
        this._rotateRadiansCos = Math.cos(this._rotateRadians);
        this._rotateRadiansSin = Math.sin(this._rotateRadians);

        this._rotationChanged();
    }

    this._setScale = function (sx, sy) {
        this._scaleX = sx;
        this._scaleY = sy;

        this._transformedHalfWidth = (this._halfWidth * this._sourceScaleX) * sx;
        this._transformedHalfHeight = (this._halfHeight * this._sourceScaleY) * sy;
        this._transformedWidth = this._transformedHalfWidth * 2;
        this._transformedHeight = this._transformedHalfHeight * 2;
    }

    this._rotationChanged = function () {

    }

    this._render = function (context, camera, offsetX, offsetY) {

    }

    this._update = function () {
        if (this._initDelay > 0) {
            this._initDelay--;
        }
        else if (this._isVisible) {
            if (this._isMoveable) {
                if (this._onPreMove != null) {
                    this._onPreMove(this);
                }

                this._x += (this._direction._x * this._speed);
                this._y += (this._direction._y * this._speed);

                if (this._onMoved != null) {
                    if (this._onMoved(this)) {
                        return false;
                    }
                }

                if (this._boundingBox != null) {
                    switch (this._clipAction) {
                        case 1:
                            if (this._x <= this._boundingBox._x - this._width || this._x >= this._boundingBox._right || this._y <= this._boundingBox._y - this._height || this._y >= this._boundingBox._bottom) {
                                this._remove(this);
                                return false;
                            }
                            break;
                        case 2:
                            if (this._direction._x < 0 && this._x <= this._boundingBox._x) {
                                this._direction._x = -this._direction._x;
                                this._x = this._boundingBox._x;
                            }
                            else if (this._direction._x > 0 && this._x >= this._boundingBox._right - this._width) {
                                this._direction._x = -this._direction._x;
                                this._x = this._boundingBox._right - this._width;
                            }

                            if (this._direction._y < 0 && this._y <= this._boundingBox._y) {
                                this._direction._y = -this._direction._y;
                                this._y = this._boundingBox._y;
                            }
                            else if (this._direction._y > 0 && this._y >= this._boundingBox._bottom - this._height) {
                                this._direction._y = -this._direction._y;
                                this._y = this._boundingBox._bottom - this._height;
                            }
                            break;
                        case 3:
                            if (this._direction._x < 0 && this._x - this._width <= this._boundingBox._x) {
                                this._x = this._boundingBox._right;
                            }
                            else if (this._direction._x > 0 && this._x >= this._boundingBox._right) {
                                this._x = this._boundingBox._x - this._width;
                            }

                            if (this._direction._y < 0 && this._y <= this._boundingBox._y - this._height) {
                                this._y = this._boundingBox._bottom;
                            }
                            else if (this._direction._y > 0 && this._y >= this._boundingBox._bottom) {
                                this._y = this._boundingBox._y - this._height;
                            }
                            break;
                        case 4:
                            if (this._direction._x < 0 && this._x <= this._boundingBox._x) {
                                this._x = this._boundingBox._x;
                            }
                            else if (this._direction._x > 0 && this._x >= this._boundingBox._right - this._width) {
                                this._x = this._boundingBox._right - this._width;
                            }

                            if (this._direction._y < 0 && this._y <= this._boundingBox._y) {
                                this._y = this._boundingBox._y;
                            }
                            else if (this._direction._y > 0 && this._y >= this._boundingBox._bottom - this._height) {
                                this._y = this._boundingBox._bottom - this._height;
                            }
                            break;
                    }
                }
            }
        }

        if (this._alphaFadeoutDelay <= 0) {
            if (this._alphaFadeout > 0) {
                this._alphaFadeout--;
                this._opacity -= this._alphaFadeoutStep;
                if (this._alphaFadeout <= 0) {
                    if (this._onFadeoutComplete != null) {
                        this._onFadeoutComplete(this);
                    }
                    if (this._removeOnFadeout) {
                        return false;
                    }
                }
            }
        }
        else {
            this._alphaFadeoutDelay--;
        }

        if (this._alphaFadeinDelay <= 0) {
            if (this._alphaFadein > 0) {
                this._alphaFadein--;
                this._opacity += this._alphaFadeinStep;
            }
        }
        else {
            this._alphaFadeinDelay--;
        }

        if (this._onUpdated != null) {
            return this._onUpdated(this);
        }

        return true;
    }

    this._fadeOut = function (delay, length, fadeOutComplete) {
        this._alphaFadeoutDelay = delay;
        this._alphaFadeoutStep = this._opacity / length;
        this._alphaFadeout = length;
        this._onFadeoutComplete = fadeOutComplete;
    }

    this._fadeIn = function (delay, length) {
        this._alphaFadeinDelay = delay;
        this._alphaFadeinStep = 1 / length;
        this._alphaFadein = length;
        this._opacity = 0;
    }

    this._remove = function () {
        if (this._onRemove != null) {
            this._onRemove(this);
        }
    }

    this._setMovement = function (xDir, yDir, speed) {
        this._isMoveable = true;
        this._direction._x = xDir;
        this._direction._y = yDir;
        this._speed = speed;
    }
}

function VectorElementAnimation() {
    this._id = 0;
    this._sequence = new Array();
    this._required = new Array();
    this._doLoop = false;
    this._currentFrame = 0;
    this._delay = 0;
    this._ticksTillNextFrame = 0;
    this._autoRun = false;
    this._completeCallback = null;

    this._dispose = function () {
        this._sequence = null;
        this._required = null;
    }

    this._clone = function () {
        var a = new VectorElementAnimation();
        a._id = this._id;
        a._frames = this._frames;
        a._sequence = this._sequence;
        a._required = this._required;
        a._doLoop = this._doLoop;
        a._delay = this._delay;
        a._ticksTillNextFrame = this._delay;
        a._autoRun = this._autoRun;

        return a;
    }

    this._nextFrame = function () {
        this._ticksTillNextFrame--;
        if (this._ticksTillNextFrame <= 0) {
            this._ticksTillNextFrame = this._delay;

            this._currentFrame++;
            if (this._currentFrame >= this._sequence.length) {
                if (this._doLoop) {
                    this._currentFrame = 0;
                }
                else {
                    this._currentFrame = this._sequence.length - 1;
                    if (this._completeCallback != null) {
                        this._completeCallback();
                    }
                }
            }
        }
    }
}

function VectorElementGroup() {
    this._inheritFrom = MoveableElement;
    this._inheritFrom();

    this._animations = null;
    this._currentAnimation = null;
    this._currentAnimationID = "";

    this._zIndex = 0;
    this._x = 0;
    this._y = 0;
    this._children = new Array();
    this._name = "";
    this._opacity = 1;
    this._type = "VectorElementGroup";

    this._onRenderComplete = null;
    this._onPreMove = null;
    this._onUpdated = null;
    this._rotate = 0;
    this._rotateCenterX = 0;
    this._rotateCenterY = 0;

    this._minWidth = 0;
    this._maxWidth = 0;
    this._minHeight = 0;
    this._maxHeight = 0;
    this._bidDim = 0;

    this._isVisible = true;
    this._isMoveable = false;
    this._direction = new Point2(0, 0);
    this._speed = 0;

    this._useTargetX = false;
    this._targetX = 0;

    this._removeNextFrame = false;
    this._isStopped = false;

    this._hitFace = null;
    this._hitPoints = null;

    this._dep = null;
    this._opacityDir = 0;
    this._setComplete = function () {
        this._opacityDir = 0.05;
    }

    this._addedToWorld = function () {
    }

    this._addAnimation = function (a) {
        if (this._animations == null) {
            this._animations = new Array();
        }

        this._animations.push(a);
    }

    this._getAnimation = function (id) {
        if (this._animations == null) {
            return null;
        }

        for (var i = 0; i < this._animations.length; i++) {
            if (this._animations[i]._id == id) {
                return this._animations[i];
            }
        }

        return null;
    }

    this._play = function (id, cb) {
        this._currentAnimation = this._getAnimation(id);
        this._currentAnimation._currentFrame = 0;
        this._currentAnimation._completeCallback = cb;
        this._currentAnimationID = id;
    }

    this._clone = function () {
        return this._cloneChildrenVectorElementGroup(new VectorElementGroup());
    }

    this._cloneChildrenVectorElementGroup = function (c) {
        c._type = this._type;
        c._name = this._name;
        c._minWidth = this._minWidth;
        c._maxWidth = this._maxWidth;
        c._minHeight = this._minHeight;
        c._maxHeight = this._maxHeight;
        c._bidDim = this._bidDim;
        c._zIndex = this._zIndex;

        for (var i = 0; i < this._children.length; i++) {
            var cc = this._children[i]._clone();
            cc._parent = c;
            c._children.push(cc);
        }

        if (this._animations != null) {
            for (var i = 0; i < this._animations.length; i++) {
                var a = this._animations[i]._clone();
                c._addAnimation(a);

                if (a._autoRun) {
                    c._currentAnimation = a;
                }
            }
        }

        return c;
    }

    this._cloneChildrenVectorElementGroupShallow = function (c) {
        c._type = this._type;
        c._name = this._name;
        c._minWidth = this._minWidth;
        c._maxWidth = this._maxWidth;
        c._minHeight = this._minHeight;
        c._maxHeight = this._maxHeight;
        c._zIndex = this._zIndex;

        for (var i = 0; i < this._children.length; i++) {
            var cc = this._children[i]._cloneShallow();
            cc._parent = c;
            c._children.push(cc);
        }

        if (this._animations != null) {
            for (var i = 0; i < this._animations.length; i++) {
                var a = this._animations[i]._clone();
                c._addAnimation(a);

                if (a._autoRun) {
                    c._currentAnimation = a;
                }
            }
        }

        return c;
    }

    this._dispose = function () {
        this._dep = null;
        this._onRenderComplete = null;
        this._onPreMove = null;
        this._onUpdated = null;
        this._hitFace = null;
        this._hitPoints = null;

        for (var i = 0; i < this._children.length; i++) {
            this._children[i]._dispose();
        }

        if (this._animations != null) {
            for (var i = 0; i < this._animations.length; i++) {
                this._animations[i]._dispose();
            }
        }

        this._children[i] = null;
        this._currentAnimation = null;
    }

    this._sourceTransformChanged = function (sx, sy) {
        this._updateHitPoints(null);
    }

    this._updateHitPoints = function (cf) {
        var o = this._children[0];

        if (cf == null && this._currentAnimation) {
            var a = this._currentAnimation;

            var s = a._sequence[a._currentFrame];
            if (s.length > 0) {
                o = this._children[s[0]];
            }
        }

        if (cf != null) {
            o = this._children[cf];
        }

        if (o._hitTestFaceList) {
            var hf = o._faces[o._hitTestFaceList[0]];
            this._hitFace = hf;
            this._hitPoints = new Array();

            if (hf) {
                if (hf._hitTestMode == 1) {
                    for (var i = 0; i < hf._points.length; i++) {
                        this._hitPoints.push(Common_RotatePoint2AroundOrigin((o._x + hf._points[i]._x) * 1, (o._y + hf._points[i]._y) * 1, o._rotateRadians, 0, 0));
                    }
                }
                else if(hf._hitTestMode == 2) {
                    this._hitPoints.push(Common_RotatePoint2AroundOrigin((o._x + -hf._halfWidth), (o._y + -hf._halfHeight), o._rotateRadians, 0, 0));
                    this._hitPoints.push(Common_RotatePoint2AroundOrigin((o._x + hf._halfWidth), (o._y + -hf._halfHeight), o._rotateRadians, 0, 0));
                    this._hitPoints.push(Common_RotatePoint2AroundOrigin((o._x + hf._halfWidth), (o._y + hf._halfHeight), o._rotateRadians, 0, 0));
                    this._hitPoints.push(Common_RotatePoint2AroundOrigin((o._x + -hf._halfWidth), (o._y + hf._halfHeight), o._rotateRadians, 0, 0));
                }
            }
        }
    }

    this._testPoints = function (p) {
        var r = false;
        for (var i = 0; i < p.length; i++) {
            r |= isPointInsidePolygon(this._x, this._y, this._hitPoints, p[i]);
        }
        return r;
    }

    this._testPoint = function (p) {
        var f = this._children[0]._faces[0];

        if (p._x >= this._x - f._halfWidth && p._x < this._x + f._halfWidth && p._y >= this._y - f._halfHeight && p._y < this._y + f._halfHeight) {
            return isPointInsidePolygon(this._x, this._y, this._hitPoints, p);
        }
        else {
        }

        return false;
    }

    this._testPointInBox = function (p) {
        var f = this._children[0]._faces[0];

        if (p._x >= this._x - f._halfWidth && p._x < this._x + f._halfWidth && p._y >= this._y - f._halfHeight && p._y < this._y + f._halfHeight) {
            return true;
        }

        return false;
    }

    this._testPointInBox2 = function (x, y) {
        var f = this._children[0]._faces[0];

        if (x >= this._x - f._halfWidth && x < this._x + f._halfWidth && y >= this._y - f._halfHeight && y < this._y + f._halfHeight) {
            return true;
        }

        return false;
    }

    this._debugDrawHitPoints = function () {
        if (this._hitPoints) {
            var c = _view._world._camera;
            for (var i = 0; i < this._hitPoints.length; i++) {
                var x = (this._hitPoints[i]._x + (this._x - c._x)) * _screenScale;
                var y = (this._hitPoints[i]._y + (this._y - c._y)) * _screenScale;

                _view._world._drawPoint(backContext, x, y);
            }
        }
    }

    this._debugDrawHitPoint = function (x, y) {
        var c = _view._world._camera;
        x = (x + (this._x - c._x)) * _screenScale;
        y = (y + (this._y - c._y)) * _screenScale;

        _view._world._drawPoint(backContext, x, y);
    }

    this._setPosition = function (x, y) {
        this._x = x;
        this._y = y;
    }

    this._show = function () {
        this._setOpacity(1);
        this._isVisible = true;
    }

    this._setOpacity = function (o) {
        for (var a = 0; a < this._children.length; a++) {
            this._children[a]._opacity = o;
        }
    }

    this._setRotate = function (o) {
        for (var a = 0; a < this._children.length; a++) {
            this._children[a]._setRotate(o);
        }
    }

    this._setScale = function (sx, sy) {
        for (var a = 0; a < this._children.length; a++) {
            this._children[a]._setScale(sx, sy);
        }
    }

    this._setScale2 = function (sx, sy) {
        for (var a = 0; a < this._children.length; a++) {
            this._children[a]._scaleX = sx;
            this._children[a]._scaleY = sy;
        }
    }

    this._render = function (context, camera, offsetX, offsetY) {
        if (this._isVisible) {
            if (this._currentAnimation) {
                var a = this._currentAnimation;

                var s = a._sequence[a._currentFrame];
                for (var i = 0; i < s.length; i++) {
                    var child = this._children[s[i]];
                    if (child._isVisible) {
                        child._render(context, camera, (offsetX + this._x), (offsetY + this._y));
                    }
                }

                for (var i = 0; i < a._required.length; i++) {
                    var child = this._children[a._required[i]];
                    if (child._isVisible) {
                        child._render(context, camera, (offsetX + this._x), (offsetY + this._y));
                    }
                }

                a._nextFrame();
            }
            else {
                for (var i = 0; i < this._children.length; i++) {
                    var child = this._children[i];
                    if (child._isVisible) {
                        child._render(context, camera, (offsetX + this._x), (offsetY + this._y));
                    }
                }
            }
        }
    }

    this._update = function () {
        if (this._removeNextFrame) {
            this._removeNextFrame = false;
            return false;
        }
        if (!this._updateMoveable()) {
            return false;
        }

        for (var i = this._children.length - 1; i >= 0; i--) {
            var child = this._children[i];
            if (!child._update()) {
                this._children.splice(i, 1);
            }

            if (this._children.length == 0) {
                return false;
            }
        }

        if (this._isMoveable && !this._isStopped) {
            this._x += (this._direction._x * this._speed);
            this._y += (this._direction._y * this._speed);
        }

        if (this._onUpdated) {
            return this._onUpdated(this);
        }

        if (this._opacityDir != 0) {
            this._opacity += this._opacityDir;
            if (this._opacity >= 1) {
                this._opacityDir = -this._opacityDir;
                this._opacity = 1;
            }
            else if (this._opacity <= 0.3) {
                this._opacityDir = -this._opacityDir;
                this._opacity = 0.3;
            }
            this._setOpacity(this._opacity);
        }

        return true;
    }

    this._setMovement = function (xDir, yDir, speed) {
        this._isMoveable = true;
        this._direction._x = xDir;
        this._direction._y = yDir;
        this._speed = speed;
    }

    this._updateToTarget = function () {
        if (this._targetRotate != 0) {
            var m = 4;
            var ma = 40;

            if (this._rotate < this._targetRotate - m) {
                var d = this._targetRotate - this._rotate;
                this._rotate += m;
            }
            else if (this._rotate > this._targetRotate + m) {
                var d = this._rotate - this._targetRotate;
                this._rotate -= m;
            }
            else {
                this._targetRotate = 0;
            }
        }
        else {
            if (this._rotate != 0) {
                this._rotate *= 0.7;
            }
        }
    }

    this._fadeOut = function (delay, length, fadeOutComplete, removeOnFadeout) {
        var th = this;

        for (var i = 0; i < this._children.length; i++) {
            var child = this._children[i];
            child._fadeOut(delay, length, null);
            if (i == 0) {
                child._fadeOut(delay, length, function () {
                    if (removeOnFadeout) {
                        th._removeNextFrame = true;
                    }
                    if (fadeOutComplete) {
                        fadeOutComplete(th);
                    }
                });
            }
            else {
                child._fadeOut(delay, length, null);
            }
        }
    }

    this._fadeIn = function (delay, length) {
        for (var i = 0; i < this._children.length; i++) {
            var child = this._children[i];
            child._fadeIn(delay, length);
        }
    }

    this._updateTransformedPoints = function () {
        this._children[0]._updateTransformedPoints();
    }
}

function SingleVectorElement(width, height, cacheCopy, cacheCopyElement) {
    this._inheritFrom = BaseElement;
    this._inheritFrom();

    this._faces = new Array();
    this._cacheCopyElement = cacheCopyElement;
    this._cacheCopy = cacheCopy;
    this._cacheContext = null;
    this._parent = null;
    this._enableCaching = true;
    this._usingSharedCachedCopy = (cacheCopy != null);

    this._width = (width >> 0);
    this._height = (height >> 0);
    this._halfWidth = this._width * 0.5;
    this._halfHeight = this._height * 0.5;

    this._sourceScaleX = 1;
    this._sourceScaleY = 1;
    this._hitTestFaceList = new Array();

    this._cloneElement = function () {
        return new SingleVectorElement(this._width, this._height, (this._enableCaching ? this._cacheCopy : null), (this._enableCaching ? this : null));
    }

    this._clone = function () {
        var c = this._cloneElement();

        c._enableCaching = this._enableCaching;
        c._x = this._x;
        c._y = this._y;
        c._setRotate(0);
        c._hitTestFaceList = this._hitTestFaceList;
        this._cloneFaces(c);

        if (!this._enableCaching) {
            c._setSourceTransforms(this._sourceScaleX, this._sourceScaleY);
            c._setScale(1, 1);
        }
        else {
            c._transformedHalfWidth = this._transformedHalfWidth;
            c._transformedHalfHeight = this._transformedHalfHeight;
            c._transformedWidth = this._transformedWidth;
            c._transformedHeight = this._transformedHeight;
        }

        c._render = this._render;

        if (!c._enableCaching) {
            //c._redraw();
        }

        return c;
    }

    this._cloneShallowElement = function () {
        return new SingleVectorElement(this._width, this._height, null, null);
    }

    this._cloneShallow = function () {
        var c = this._cloneShallowElement();

        this._cloneFaces(c);

        c._transformedHalfWidth = this._transformedHalfWidth;
        c._transformedHalfHeight = this._transformedHalfHeight;
        c._transformedWidth = this._transformedWidth;
        c._transformedHeight = this._transformedHeight;
        c._render = this._render;

        return c;
    }

    this._cloneFaces = function (c) {
        for (var i = 0; i < this._faces.length; i++) {
            c._faces.push(this._faces[i]._clone());
        }

        return c;
    }

    this._dispose = function () {
        this._cacheCopyElement = null;
        this._cacheCopy = null;
        this._cacheContext = null;
        this._parent = null;

        if (this._faces != null) {
            for (var i = 0; i < this._faces.length; i++) {
                if (this._faces[i] != null) {
                    this._faces[i]._dispose();
                }
            }
        }

        this._faces = null;
    }

    this._setSourceTransforms = function (scaleX, scaleY) {
        this._setSourceTransformsNoRebuild(scaleX, scaleY);

        this._rebuildCache();
    }

    this._setSourceTransformsNoRebuild = function (scaleX, scaleY) {
        this._sourceScaleX = scaleX;
        this._sourceScaleY = scaleY;
        this._transformedHalfWidth = (this._halfWidth * this._sourceScaleX);
        this._transformedHalfHeight = (this._halfHeight * this._sourceScaleY);
        this._transformedWidth = this._transformedHalfWidth * 2;
        this._transformedHeight = this._transformedHalfHeight * 2;
    }

    this._render = function (context, camera, offsetX, offsetY) {
    }

    this._renderMode1 = function (context, camera, offsetX, offsetY) {
        context.save();
        var tx = (this._x + (offsetX - camera._x)) * _screenScale;
        var ty = (this._y + (offsetY - camera._y)) * _screenScale;
        context.globalAlpha = this._opacity;

        context.drawImage(this._cacheCopy, ((0.5 + (tx - this._transformedHalfWidth)) >> 0), ((0.5 + (ty - this._transformedHalfHeight)) >> 0));
        context.restore();
    }

    this._renderMode2 = function (context, camera, offsetX, offsetY) {
        if (this._isVisible) {
            context.save();
            var tx = (offsetX - camera._x) * _screenScale;
            var ty = (offsetY - camera._y) * _screenScale;

            context.globalAlpha = this._opacity;
            context.translate(tx + this._parent._rotateCenterX, ty + this._parent._rotateCenterY);
            context.rotate(this._parent._rotate * 0.0174532925199432957);
            context.translate(-this._parent._rotateCenterX, -this._parent._rotateCenterY);
            context.translate((this._x * _screenScale) - this._transformedHalfWidth, (this._y * _screenScale) - this._transformedHalfHeight);

            context.translate(this._transformedHalfWidth, this._transformedHalfHeight);
            context.rotate(this._rotate * 0.0174532925199432957);
            context.scale(this._scaleX, this._scaleY);
            context.translate(-this._transformedHalfWidth, -this._transformedHalfHeight);

            context.drawImage(this._cacheCopy, 0, 0);
            context.restore();
        }
    }

    this._renderFillAndStroke = function (context, offsetX, offsetY, scaleX, scaleY) {
        context.save();

        //context.fillStyle = "rgb(255,0,0)";
        //context.fillRect(0, 0, this._width * scaleX, this._height * scaleY);

        for (var i = 0; i < this._faces.length; i++) {
            this._faces[i]._renderFillAndStroke(context, offsetX, offsetY, scaleX, scaleY, this._width, this._height);
        }

        context.restore();
    }

    this._renderStrokeOnly = function (context, offsetX, offsetY, scaleX, scaleY, w, c, f, cr, cric) {
        context.save();

        if (cr != null) {
            context.save();
            context.beginPath();
            context.rect(cr._x, cr._y, cr._width, cr._height);
            context.clip();
        }

        for (var i = 0; i < this._faces.length; i++) {
            if (cr != null && i >= cric) {
                context.restore();
            }

            this._faces[i]._renderStrokeOnly(context, offsetX, offsetY, scaleX, scaleY, this._width, this._height, w, c, f);
        }

        context.restore();
    }

    this._initCache = function () {
        if (this._cacheCopy == null && !this._usingSharedCachedCopy) {
            this._cacheCopy = document.createElement("canvas");

            this._cacheCopy.width = Math.ceil(this._width * this._sourceScaleX) + "";
            this._cacheCopy.height = Math.ceil(this._height * this._sourceScaleY) + "";

            this._cacheContext = this._cacheCopy.getContext('2d');
        }
    }

    this._applyNewCacheCanvas = function (c) {
        if (c == null) {
            c = this._cacheCopyElement._cacheCopy;
        }

        if (c != null) {
            this._cacheCopy = c;
            this._cacheContext = this._cacheCopy.getContext('2d');
        }
    }

    this._redraw = function (o) {
        this._rebuildCache(o);
    }

    this._rebuildCache = function (o) {
        if (!this._usingSharedCachedCopy) {
            this._cacheContext = null;
            this._cacheCopy = null;

            this._initCache();
            this._updateCache();
        }
        else {
            if (this._cacheCopyElement != null) {
                this._cacheCopy = this._cacheCopyElement._cacheCopy;
                this._cacheContext = this._cacheCopy.getContext('2d');
            }
            else if (o) {
                this._cacheContext = this._cacheCopy.getContext('2d');
                this._updateCache();
            }
            else {
                this._updateTransformedPoints();
            }
        }
    }

    this._updateCache = function () {
        this._updateTransformedPoints();
        this._renderFillAndStroke(this._cacheContext, this._halfWidth, this._halfHeight, this._sourceScaleX, this._sourceScaleY);
    }

    this._redrawStrokeOnly = function (w, c, f, cr, cric) {
        this._cacheContext.clearRect(0, 0, this._width * this._sourceScaleX, this._height * this._sourceScaleY);

        this._updateTransformedPoints();
        this._renderStrokeOnly(this._cacheContext, this._halfWidth, this._halfHeight, this._sourceScaleX, this._sourceScaleY, w, c, f, cr, cric);
    }

    this._updateTransformedPoints = function () {
        var i;
        var a;

        for (a = 0; a < this._faces.length; a++) {
            var shape = this._faces[a];
            //if (shape._isCollidable) {
            if (shape._points != null) {
                shape._transformedPoints = new Array();

                for (i = 0; i < shape._points.length; i++) {
                    var p = shape._points[i];
                    var v = new Vector2(this._x + ((this._rotateRadiansCos * p._x - this._rotateRadiansSin * p._y) * this._sourceScaleX), this._y + ((this._rotateRadiansSin * p._x + this._rotateRadiansCos * p._y) * this._sourceScaleY));

                    shape._transformedPoints.push(v);
                }
                shape._buildEdges();
            }
            //}
        }
    }

    //this._initCache();
}

function BaseShape() {
    this._type = 0;
    this._fillStyle = null;
    this._strokeStyle = null;
    this._strokeWidth = 0;
    this._shadowColor = null;
    this._shadowBlur = 4;
    this._shadowOffsetX = 1;
    this._shadowOffsetY = 1;
    this._minX = 0;
    this._minY = 0;
    this._maxX = 0;
    this._maxY = 0;
    this._width = 0;
    this._height = 0;
    this._halfWidth = 0;
    this._halfHeight = 0;
    this._longDim = 0;
    this._isCollidable = false;
    this._isVisible = true;
    this._hitTestMode = 0;

    this._clone = function () {
    }

    this._cloneBaseShape = function (c) {
        c._type = this._type;
        c._fillStyle = this._fillStyle;
        c._strokeStyle = this._strokeStyle;
        c._strokeWidth = this._strokeWidth;
        c._minX = this._minX;
        c._minY = this._minY;
        c._maxX = this._maxX;
        c._maxY = this._maxY;
        c._width = this._width;
        c._height = this._height;
        c._halfWidth = this._halfWidth;
        c._halfHeight = this._halfHeight;
        c._longDim = this._longDim;
        c._isCollidable = this._isCollidable;
        c._isVisible = this._isVisible;
        c._hitTestMode = this._hitTestMode;

        return c;
    }

    this._dispose = function () {
        this._fillStyle = null;
        this._strokeStyle = null;
    }

    this._setStyles = function (fill, stroke, strokeWidth) {
        this._fillStyle = fill;
        this._strokeStyle = stroke;
        this._strokeWidth = strokeWidth;
    }

    this._setStylesFromFace = function (face) {
        this._fillStyle = face._fillStyle;
        this._strokeStyle = face._strokeStyle;
        this._strokeWidth = face._strokeWidth;
    }

    this._updateTransformedPoints = function () {
    }

    this._buildEdges = function () {
    }

    this._getCenter = function () {
    }

    this._createBrush = function (context, faceStyle, offsetX, offsetY, scaleX, scaleY, radius) {
        var brush = null;

        if (faceStyle == null) {
            return null;
        }

        var modelWidth = offsetX * 2;
        var modelHeight = offsetY * 2;
        var sX = this._minX + offsetX;
        var sY = this._minY + offsetY;

        switch (faceStyle._type) {
            case "image":
                brush = faceStyle._obj;
                break;
            case "linear":
                var w = faceStyle._width;
                var h = faceStyle._height;

                if (w != null) {
                    sX = (-w * 0.5) + offsetX;
                }
                else {
                    w = this._width;
                }
                if (h != null) {
                    sY = (-h * 0.5) + offsetY;
                }
                else {
                    h = this._height;
                }

                var x0 = (((sX + (faceStyle._obj._x0 * w)) * scaleX) >> 0);
                var y0 = (((sY + (faceStyle._obj._y0 * h)) * scaleY) >> 0);
                var x1 = (((sX + (faceStyle._obj._x1 * w)) * scaleX) >> 0);
                var y1 = (((sY + (faceStyle._obj._y1 * h)) * scaleY) >> 0);

                brush = context.createLinearGradient(x0, y0, x1, y1);

                for (var i = 0; i < faceStyle._obj._steps.length; i++) {
                    brush.addColorStop(faceStyle._obj._steps[i]._start, faceStyle._obj._steps[i]._color);
                }
                break;
            case "radial":
                var sX = (this._minX + offsetX) * scaleX;
                var sY = (this._minY + offsetY) * scaleY;
                var cX0 = sX + ((faceStyle._obj._x0 * radius) * scaleX);
                var cY0 = sY + ((faceStyle._obj._y0 * radius) * scaleY);
                var cX1 = sX + ((faceStyle._obj._x0 * radius) * scaleX);
                var cY1 = sY + ((faceStyle._obj._y0 * radius) * scaleY);

                brush = context.createRadialGradient(cX0, cY0, 0, cX1, cY1, radius * scaleX);

                for (var i = 0; i < faceStyle._obj._steps.length; i++) {
                    brush.addColorStop(faceStyle._obj._steps[i]._start, faceStyle._obj._steps[i]._color);
                }
                break;
            case "solid":
                brush = faceStyle._obj;
                break;
        }

        return brush;
    }
}

function EffectShape() {
    this._inheritFrom = BaseShape;
    this._inheritFrom();

    this._effectJs = "";

    this._renderFillAndStroke = function (c, ox, oy, sx, sy, cw, ch) {
        eval(this._effectJs);
    }
}

function CircleShape() {
    this._inheritFrom = BaseShape;
    this._inheritFrom();

    this._type = 1;
    this._x = 0;
    this._y = 0;
    this._radius = 10;

    this._clone = function () {
        var c = new CircleShape();
        this._cloneBaseShape(c);

        c._type = this._type;
        c._x = this._x;
        c._y = this._y;
        c._radius = this._radius;

        return c;
    }

    this._renderFillAndStroke = function (context, offsetX, offsetY, scaleX, scaleY, cw, ch) {
        if (this._isVisible) {
            context.fillStyle = this._createBrush(context, this._fillStyle, offsetX - this._radius, offsetY - this._radius, scaleX, scaleY, this._radius);

            if (this._strokeWidth > 0) {
                var sw = (this._strokeWidth * scaleX) >> 0;
                if (sw <= 0) {
                    sw = 1;
                }

                context.lineWidth = sw;
                context.strokeStyle = this._createBrush(context, this._strokeStyle, offsetX - this._radius, offsetY - this._radius, scaleX, scaleY, this._radius);
            }

            var hR = this._radius;

            context.beginPath();
            context.arc(((this._x + offsetX) * scaleX), ((this._y + offsetY) * scaleY), this._radius * scaleX, 0, pi2, false);
            context.closePath();

            if (this._strokeWidth > 0) {
                context.stroke();
            }

            context.fill();
        }
    }

    this._renderStrokeOnly = function (context, offsetX, offsetY, scaleX, scaleY, cw, ch, w, c, f) {
        if (this._isVisible) {
            context.lineWidth = w;
            context.strokeStyle = c;
            context.fillStyle = f;

            var hR = this._radius;

            context.beginPath();
            context.arc(((this._x + offsetX) * scaleX), ((this._y + offsetY) * scaleY), this._radius * scaleX, 0, pi2, false);
            context.closePath();

            context.stroke();
            context.fill();
        }
    }
}

function TextBlock() {
    this._inheritFrom = BaseShape;
    this._inheritFrom();

    this._x = 0;
    this._y = 0;
    this._fontFamily = "Arial";
    this._fontSize = 12;
    this._text = "";
    this._hAlign = 0;
    this._vAlign = 0;

    this._clone = function () {
        var c = new TextBlock();
        this._cloneBaseShape(c);

        c._x = this._x;
        c._y = this._y;
        c._fontFamily = this._fontFamily;
        c._fontSize = this._fontSize;
        c._text = this._text;
        c._hAlign = this._hAlign;
        c._vAlign = this._vAlign;

        return c;
    }

    this._renderStrokeOnly = function (context, offsetX, offsetY, scaleX, scaleY, cw, ch, w, c, f) {
        this._renderFillAndStroke(context, offsetX, offsetY, scaleX, scaleY, cw, ch);
    }

    this._renderFillAndStroke = function (context, offsetX, offsetY, scaleX, scaleY, cw, ch) {
        if (this._isVisible) {
            context.fillStyle = this._createBrush(context, this._fillStyle, offsetX - this._longDim, offsetY - this._longDim, scaleX, scaleY, this._longDim);

            var fs = (this._fontSize * scaleX) >> 0;
            /*if (this._strokeStyle._type != null) {0
            context.lineWidth = this._strokeWidth;
            context.strokeStyle = this._createBrush(context, this._strokeStyle, offsetX, offsetY, scaleX, scaleY, this._longDim);
            }*/

            if (this._shadowColor != null) {
                context.shadowColor = this._shadowColor;
                context.shadowBlur = this._shadowBlur;
                context.shadowOffsetX = this._shadowOffsetX;
                context.shadowOffsetY = this._shadowOffsetY;
            }

            context.font = fs + "px " + this._fontFamily;
            var x = 0;
            var y = 0;
            var s = context.measureText(this._text);
            ch *= scaleY;
            cw *= scaleX;

            if (this._hAlign == 0) {
                x = 0;
            }
            else if (this._hAlign == 1) {
                x = (cw - s.width) >> 1;
            }
            else if (this._hAlign == 2) {
                x = cw - s.width;
            }

            if (this._vAlign == 0) {
                y = fs;
            }
            else if (this._vAlign == 1) {
                y = ((ch - fs) >> 1) + fs;
            }
            else if (this._vAlign == 2) {
                y = ch - fs;
            }

            x = (this._x * scaleX) + x;
            y = (this._y * scaleY) + y;

            context.fillText(this._text, x, y);
        }
    }
}

function PointShape() {
    this._inheritFrom = BaseShape;
    this._inheritFrom();

    this._type = 2;
    this._points = new Array();
    this._transformedPoints = new Array();
    this._edges = new Array();

    this._clonePointShape = function (c) {
        this._cloneBaseShape(c);

        for (var i = 0; i < this._points.length; i++) {
            c._points.push(this._points[i]._clone());
        }
        this._updateTransformedPoints();

        return c;
    }

    this._dispose = function () {
        this._fillStyle = null;
        this._strokeStyle = null;
        this._points = null;
        this._transformedPoints = null;
        this._edges = null;
    }

    this._updateTransformedPoints = function () {
        var i;
        this._transformedPoints = new Array();

        for (i = 0; i < this._points.length; i++) {
            var p = this._points[i];
            var v = new Vector2(this._x + (this._rotateRadiansCos * p._x - this._rotateRadiansSin * p._y), this._y + (this._rotateRadiansSin * p._x + this._rotateRadiansCos * p._y));
            this._transformedPoints.push(v);
        }

        this._buildEdges();
    }

    this._buildEdges = function () {
        var p1;
        var p2;
        this._edges = new Array();
        for (var i = 0; i < this._transformedPoints.length; i++) {
            p1 = this._transformedPoints[i];
            if (i + 1 >= this._transformedPoints.length) {
                p2 = this._transformedPoints[0];
            }
            else {
                p2 = this._transformedPoints[i + 1];
            }
            this._edges.push(p2._subNew(p1));
        }
    }

    this._getCenter = function () {
        var totalX = 0;
        var totalY = 0;
        for (var i = 0; i < this._transformedPoints.length; i++) {
            totalX += this._transformedPoints[i]._x;
            totalY += this._transformedPoints[i]._y;
        }

        return new Vector2(totalX / this._transformedPoints.length, totalY / this._transformedPoints.length);
    }
}

function Polygon() {
    this._inheritFrom = PointShape;
    this._inheritFrom();

    this._clone = function () {
        var c = new Polygon();
        this._clonePointShape(c);
        return c;
    }

    this._renderFillAndStroke = function (context, offsetX, offsetY, scaleX, scaleY, cw, ch) {
        if (this._isVisible) {
            context.fillStyle = this._createBrush(context, this._fillStyle, offsetX, offsetY, scaleX, scaleY, this._longDim);
            if (this._strokeWidth > 0) {
                var sw = (this._strokeWidth * scaleX) >> 0;
                if (sw <= 0) {
                    sw = 1;
                }

                context.lineWidth = sw;
                context.strokeStyle = this._createBrush(context, this._strokeStyle, offsetX, offsetY, scaleX, scaleY, this._longDim);
            }

            if (this._shadowColor != null) {
                context.shadowColor = this._shadowColor;
                context.shadowBlur = this._shadowBlur;
                context.shadowOffsetX = this._shadowOffsetX;
                context.shadowOffsetY = this._shadowOffsetY;
            }

            context.beginPath();
            context.moveTo((offsetX + this._points[0]._x) * scaleX, (offsetY + this._points[0]._y) * scaleY);

            for (i = 1; i < this._points.length; i++) {
                context.lineTo((offsetX + this._points[i]._x) * scaleX, (offsetY + this._points[i]._y) * scaleY);
            }

            context.fill();

            if (this._strokeWidth > 0) {
                context.stroke();
            }
        }
    }

    this._renderStrokeOnly = function (context, offsetX, offsetY, scaleX, scaleY, cw, ch, w, c, f) {
        if (this._isVisible) {
            context.lineWidth = w;
            context.strokeStyle = c;
            context.fillStyle = f;

            context.beginPath();
            context.moveTo((offsetX + this._points[0]._x) * scaleX, (offsetY + this._points[0]._y) * scaleY);

            for (i = 1; i < this._points.length; i++) {
                context.lineTo((offsetX + this._points[i]._x) * scaleX, (offsetY + this._points[i]._y) * scaleY);
            }

            context.fill();
            context.stroke();
        }
    }
}

function BitmapShape() {
    this._inheritFrom = BaseShape;
    this._inheritFrom();

    this._x = 0;
    this._y = 0;
    this._width = 0;
    this._height = 0;
    this._halfWidth = 0;
    this._halfHeight = 0;
    this._src = "";
    this._mirror = 1;
    this._flip = 1;
    this._rotate = 0;
    this._scaleX = 1;
    this._scaleY = 1;

    this._clone = function () {
        var b = new BitmapShape();
        b._x = this._x;
        b._y = this._y;
        b._width = this._width;
        b._height = this._height;
        b._halfWidth = this._halfWidth;
        b._halfHeight = this._halfHeight;
        b._hitTestMode = this._hitTestMode;

        return b;
    }

    this._renderFillAndStroke = function (context, offsetX, offsetY, scaleX, scaleY, cw, ch) {
        if (this._isVisible) {
            var cx = this._x * scaleX;
            var cy = this._y * scaleY;
            var i = _imageManager._images[this._src];
            var hw = i.width >> 1;
            var hh = i.height >> 1;
            var sx = this._scaleX * scaleX;
            var sy = this._scaleY * scaleY;

            context.save();
            context.translate((cx - hw) >> 0, (cy - hh) >> 0);
            context.translate(hw, hh);
            context.scale(sx * this._mirror, sy * this._flip);
            context.rotate(this._rotate * 0.0174532925199432957);
            context.translate(-hw, -hh);

            context.drawImage(i, 0, 0);
            context.restore();
        }
    }
}

function GridShape() {
    this._inheritFrom = PointShape;
    this._inheritFrom();
    this._cellWidth = 1;
    this._cellHeight = 1;
    this._offFillStyle = null;
    this._offStrokeStyle = null;
    this._onStartIndex = 0;
    this._cellShape = 0;

    this._clone = function () {
        var c = new GridShape();
        c._cellWidth = this._cellWidth;
        c._cellHeight = this._cellHeight;
        c._offFillStyle = this._offFillStyle;
        c._offStrokeStyle = this._offStrokeStyle;
        c._cellShape = this._cellShape;

        this._clonePointShape(c);
        return c;
    }

    this._setOffStyles = function (fill, stroke) {
        this._offFillStyle = fill;
        this._offStrokeStyle = stroke;
    }

    this._renderFillAndStroke = function (context, offsetX, offsetY, scaleX, scaleY) {
        if (this._isVisible) {
            context.save();
            context.fillStyle = this._createBrush(context, this._offFillStyle, offsetX, offsetY, scaleX, scaleY, this._cellWidth);
            if (this._strokeWidth > 0) {
                context.lineWidth = this._strokeWidth;
                context.strokeStyle = this._createBrush(context, this._offStrokeStyle, offsetX, offsetY, scaleX, scaleY, this._cellWidth);
            }

            if (this._shadowColor != null) {
                context.shadowColor = this._shadowColor;
                context.shadowBlur = this._shadowBlur;
                context.shadowOffsetX = this._shadowOffsetX;
                context.shadowOffsetY = this._shadowOffsetY;
            }

            for (var i = 0; i < this._points.length; i++) {
                if (i == this._onStartIndex) {
                    context.fillStyle = this._createBrush(context, this._fillStyle, offsetX, offsetY, scaleX, scaleY, this._cellWidth);
                    if (this._strokeWidth > 0) {
                        context.lineWidth = this._strokeWidth;
                        context.strokeStyle = this._createBrush(context, this._strokeStyle, offsetX, offsetY, scaleX, scaleY, this._cellWidth);
                    }
                }

                var hcw = this._cellWidth * 0.5;
                var hch = this._cellHeight * 0.5;
                var x = (offsetX + ((this._points[i]._x * this._cellWidth) - hcw)) * scaleX;
                var y = (offsetY + ((this._points[i]._y * this._cellHeight) - hch)) * scaleY;
                
                context.fillRect(x, y, this._cellWidth * scaleX, this._cellHeight * scaleY);
                context.strokeRect(x, y, this._cellWidth * scaleX, this._cellHeight * scaleY);
            }

            context.restore();
        }
    }

    this._renderFillAndStrokeSingleItem = function (context, offsetX, offsetY, scaleX, scaleY, i) {
        if (this._isVisible) {
            context.save();

            if (i < this._onStartIndex) {
                context.fillStyle = this._createBrush(context, this._fillStyle, offsetX, offsetY, scaleX, scaleY, this._cellWidth);
                if (this._strokeWidth > 0) {
                    context.lineWidth = this._strokeWidth;
                    context.strokeStyle = this._createBrush(context, this._strokeStyle, offsetX, offsetY, scaleX, scaleY, this._cellWidth);
                }
            }
            else {
                context.fillStyle = this._createBrush(context, this._offFillStyle, offsetX, offsetY, scaleX, scaleY, this._cellWidth);
                if (this._strokeWidth > 0) {
                    context.lineWidth = this._strokeWidth;
                    context.strokeStyle = this._createBrush(context, this._offStrokeStyle, offsetX, offsetY, scaleX, scaleY, this._cellWidth);
                }
            }

            var x = (this._points[i]._x * this._cellWidth);
            var y = (this._points[i]._y * this._cellHeight);
            var hcw = this._cellWidth * 0.5;
            var hch = this._cellHeight * 0.5;

            switch (this._cellShape) {
                case 0:
                    context.beginPath();
                    context.moveTo((offsetX + (x - hcw)) * scaleX, (offsetY + (y - hch)) * scaleY);
                    context.lineTo((offsetX + (x + hcw)) * scaleX, (offsetY + (y - hch)) * scaleY);
                    context.lineTo((offsetX + (x + hcw)) * scaleX, (offsetY + (y + hch)) * scaleY);
                    context.lineTo((offsetX + (x - hcw)) * scaleX, (offsetY + (y + hch)) * scaleY);
                    context.lineTo((offsetX + (x - hcw)) * scaleX, (offsetY + (y - hch)) * scaleY);
                    break;
                case 1:
                    context.beginPath();
                    context.arc(((x + offsetX) * scaleX), ((y + offsetY) * scaleY), (hcw + 2) * scaleX, 0, pi2, false);
                    context.closePath();
                    break;
                case 2:
                    context.beginPath();
                    context.moveTo((offsetX + (x - 3)) * scaleX, (offsetY + (y - 6)) * scaleY);
                    context.lineTo((offsetX + (x + 3)) * scaleX, (offsetY + (y - 6)) * scaleY);
                    context.lineTo((offsetX + (x + 8)) * scaleX, (offsetY + y) * scaleY);
                    context.lineTo((offsetX + (x + 3)) * scaleX, (offsetY + (y + 6)) * scaleY);
                    context.lineTo((offsetX + (x - 3)) * scaleX, (offsetY + (y + 6)) * scaleY);
                    context.lineTo((offsetX + (x - 8)) * scaleX, (offsetY + y) * scaleY);
                    context.lineTo((offsetX + (x - 3)) * scaleX, (offsetY + (y - 6)) * scaleY);
                    break;
            }

            context.fill();

            if (this._strokeWidth > 0) {
                context.stroke();
            }

            context.restore();
        }
    }
}

function BoundingBox(x, y, width, height) {
    this._x = x;
    this._y = y;
    this._width = width;
    this._height = height;
    this._right = x + width;
    this._bottom = y + height;

    this._dispose = function () {
    }

    this._intersects = function (r) {
        var tw = this._width;
        var th = this._height;
        var rw = r._width;
        var rh = r._height;
        if (rw <= 0 || rh <= 0 || tw <= 0 || th <= 0) {
            return false;
        }
        var tx = this._x;
        var ty = this._y;
        var rx = r._x;
        var ry = r._y;
        rw += rx;
        rh += ry;
        tw += tx;
        th += ty;
        //      overflow || intersect
        return ((rw < rx || rw > tx) &&
                (rh < ry || rh > ty) &&
                (tw < tx || tw > rx) &&
                (th < ty || th > ry));
    }
}

function FaceStyle(type, obj, w, h) {
    this._type = type;
    this._obj = obj;
    this._width = w;
    this._height = h;
}

function Camera() {
    this._x = 0;
    this._y = 0;
    this._z = 200;
    this._zoom = 1;
    this._viewport = new Size2(0, 0);
    this._calculatedXOffset = 0;
    this._calculatedYOffset = 0;

    this._dispose = function () {
        this._viewport = null;
    }

    this._calculateOffsets = function () {
        this._calculatedXOffset = (this._viewport._width * 0.5) * -this._zoom;
        this._calculatedYOffset = (this._viewport._height * 0.5) * -this._zoom;
    }

    this._clone = function () {
        var c = new Camera();
        c._x = this._x;
        c._y = this._y;
        c._z = this._z;
        c._zoom = this._zoom;
        c._viewport = new Size2(this._viewport._width, this._viewport._height);
        c._calculatedXOffset = this._calculatedXOffset;
        c._calculatedYOffset = this._calculatedYOffset;

        return c;
    }
}

function ClippingLayer(c) {
    this._start = c;
    this._end = c;
    this._hitList = null;

    this._clone = function () {
        var c = new ClippingLayer(0);
        c._start = this._start;
        c._end = this._end;
        c._hitList = null;

        return c;
    }
}