﻿function GameUnit() {
    this._inheritFrom = VectorElementGroup;
    this._inheritFrom();

    this._index = 0;
    this._indexPart = 0;

    this._clone = function () {
        return this._cloneChildrenVectorElementGroup(new GameUnit());
    }

    this._update = function () {

        return true;
    }

    this._isInPlayingArea = function () {
        if (this._x >= AREA_MIN_X && this._x < AREA_MAX_X && this._y >= AREA_MIN_Y && this._y < AREA_MAX_Y) {
            return true;
        }

        return false;
    }
}

function Asteroid() {
    this._inheritFrom = VectorElementGroup;
    this._inheritFrom();

    this._spin = 0;
    this._direction = new Point2(0, 0);
    this._speed = 1;
    this._hasDied = false;
    this._hp = 1;

    this._clone = function () {
        return this._cloneChildrenVectorElementGroup(new Asteroid());
    }

    this._hit = function (s) {
        var h = s._strength / this._animations.length;
        this._hp += h;

        if (this._hp <= 0) {
            this._hasDied = true;
            _view._destroyAstroid(this);
        }
        else {
            var f = (this._animations.length * (1-this._hp)) >> 0;
            this._play("f" + (f + 1));
        }
    }

    this._update = function () {
        var r = this._children[0]._rotate + this._spin;
        this._setRotate(r);

        if (this._indexPart == _view._updateTickPart) {
            this._updateHitPoints(0);
        }

        this._x += this._direction._x * this._speed;
        this._y += this._direction._y * this._speed;

        if (this._x < AREA_MIN_X || this._x > AREA_MAX_X) {
            this._direction._x = -this._direction._x;
        }
        if (this._y < AREA_MIN_Y || this._y > AREA_MAX_Y) {
            this._direction._y = -this._direction._y;
        }

        if (this._hasDied) {
            return false;
        }

        return true;
    }
}

function Player() {
    this._inheritFrom = GameUnit;
    this._inheritFrom();

    this._isInit = true;
    this._paused = false;
    this._speed = -1;
    this._diedShieldTimer = 0;
    this._hasDied = false;
    this._isHuman = false;
    this._target = null;
    this._weapon = "b1";
    this._weaponCount = 0;
    this._ticksTillNextShot = 0;
    this._hp = 1;
    this._defense = 0.05;
    this._strength = -0.3;
    this._targetHp = 1;
    this._score = 0;
    this._targetScore = 0;
    this._targetAngle = 0;
    this._direction = new Point2(0, 0);
    this._flameAnimation = 3;
    this._showFlame = false;
    this._shieldAnimation = 10;
    this._showShield = false;
    this._shieldTimer = 0;
    this._fireDelay = 0;
    this._fireSpeed = 10;
    this._hitTimeout = 0;
    this._hitAlphaDir = -0.1;
    this._opacity = 1;

    this._clone = function () {
        return this._cloneChildrenVectorElementGroup(new Player());
    }

    this._gameUnitDispose = this._dispose;
    this._dispose = function () {
        this._gameUnitDispose();
    }

    this._render = function (context, camera, offsetX, offsetY) {
        var a = this._currentAnimation;
        var s = a._sequence[a._currentFrame];
        var c = this._children[s[0]];

        if (!this._isVisible) {
            return;
        }

        offsetX += this._x;
        offsetY += this._y;

        var tx = (offsetX - camera._x) * _screenScale;
        var ty = (offsetY - camera._y) * _screenScale;

        context.save();
        context.globalAlpha = this._opacity;
        context.translate(tx, ty);
        context.translate(((c._x * _screenScale) - c._transformedHalfWidth) >> 0, ((c._y * _screenScale) - c._transformedHalfHeight) >> 0);
        context.translate(c._transformedHalfWidth, c._transformedHalfHeight);
        context.rotate(c._rotate * 0.0174532925199432957);
        context.translate(-c._transformedHalfWidth, -c._transformedHalfHeight);

        context.drawImage(c._cacheCopy, 0, 0);

        if (this._showFlame) {
            y = 71 * _screenScale;
            context.drawImage(this._children[this._flameAnimation]._cacheCopy, 19 * _screenScale, y);
            context.drawImage(this._children[this._flameAnimation]._cacheCopy, 55 * _screenScale, y);
        }

        context.restore();

        if (this._showShield) {
            var c = this._children[this._shieldAnimation];
            tx += ((c._x * _screenScale) - c._transformedHalfWidth) >> 0;
            ty += ((c._y * _screenScale) - c._transformedHalfHeight) >> 0;
            context.drawImage(c._cacheCopy, tx, ty);
        }

        this._flameAnimation++;
        if (this._flameAnimation > 9) {
            this._flameAnimation = 3;
        }

        this._shieldAnimation++;
        if (this._shieldAnimation > 15) {
            this._shieldAnimation = 10;
        }
    }

    this._setShield = function () {
        this._shieldTimer = 15 * 30;
        this._showShield = true;
    }

    this._getFireAngle = function () {
        return this._children[0]._rotate;
    }

    this._addHp = function (v, d) {
        this._targetHp += v * d;

        if (this._targetHp <= 0) {
            this._targetHp = 0;
            return false;
        }
        else if (this._targetHp > 1) {
            this._targetHp = 1;
        }

        return true;
    }

    this._addScore = function (v) {
        this._targetScore += v;
    }

    this._isFunctional = function () {
        return true;
    }

    this._canFire = function () {
        return this._fireDelay <= 0;
    }

    this._hit = function (s) {
        if (!this._hasDied) {
            if (!this._addHp(s._strength, this._defense)) {
                this._died(this);
            }
        }
    }

    this._died = function (s) {
        if (!this._hasDied) {
            this._hasDied = true;
            this._isVisible = false;
            _view._unitDestroyed(this);

            _view._createExp(this, "e2", 64, 64);

            if (this._isHuman) {
                _view._gameover();
            }
        }
    }

    this._doFire = function () {
        if (this._canFire() && !_view._isGameOver) {
            this._fireDelay = this._fireSpeed;
            _view._createBullet(this._weapon, this);

            if (this._isHuman) {
                if (this._weapon != "b1") {
                    this._weaponCount--;
                    _view._updateWeaponCount();

                    if (this._weaponCount == 0) {
                        _view._setWeapon("b1");
                    }
                }
            }
        }
    }

    this._setTarget = function (x, y) {
        if (!this._isLanding) {
            this._x += x;
            this._y += y;
        }
    }

    this._update = function () {
        var d = (this._targetHp - this._hp) * 0.2;
        this._hp += d;

        if (Math.abs(this._targetHp - this._hp) < 1) {
            this._hp = this._targetHp;
        }

        d = (this._targetScore - this._score) * 0.2;
        this._score += d;

        if (Math.abs(this._targetScore - this._score) < 1) {
            this._score = this._targetScore;
        }

        if (this._shieldTimer > 0) {
            this._shieldTimer--;
            if (this._shieldTimer == 0) {
                this._showShield = false;
            }
        }

        this._fireDelay--;
        if (this._fireDelay < 0) {
            this._fireDelay = 0;
        }

        /*d = (this._targetAngle - this._children[0]._rotate) * 0.5;
        if (Math.abs(d) > 0.1) {
            this._setRotate(this._children[0]._rotate + d);
        }*/

        if (!this._hasDied) {
            this._x += this._direction._x;
            this._y += this._direction._y;

            this._children[0]._update();

            if (this._indexPart == _view._updateTickPart) {
            }

            if (this._paused) {
                return true;
            }
            //this._checkHits();
        }

        if (!_view._isGameOver) {
            this._updateHitPoints(0);
            this._checkHits();
        }

        if (this._x < AREA_MIN_X) {
            this._x = AREA_MIN_X;
            this._direction._x = 0;
        }
        else if (this._x > AREA_MAX_X) {
            this._x = AREA_MAX_X;
            this._direction._x = 0;
        }
        if (this._y < AREA_MIN_Y) {
            this._y = AREA_MIN_Y;
            this._direction._y = 0;
        }
        else if (this._y > AREA_MAX_Y) {
            this._y = AREA_MAX_Y;
            this._direction._y = 0;
        }

        return true;
    }

    this._updateCom = function() {
        if (this._indexPart == _view._updateTickPart) {
            
        }
    }

    this._checkHits = function () {
        if (this._hitTimeout > 0) {
            this._hitTimeout--;

            this._opacity += this._hitAlphaDir;
            if (this._opacity > 1) {
                this._opacity = 1;
                this._hitAlphaDir = -this._hitAlphaDir;
            }
            else if (this._opacity < 0.5) {
                this._opacity = 0.5;
                this._hitAlphaDir = -this._hitAlphaDir;
            }
            return false;
        }

        this._opacity = 1;

        var l = _view._world._layers[_view._astroidLayer];
        var p = this._hitPoints[_view._updateTickPart];
        var p2 = new Point2(p._x + this._x, p._y + this._y);

        for (var i = 0; i < l.length; i++) {
            var o = l[i];
            if (!o._hasDied) {
                if (o._testPoint(p2)) {
                    this._direction._x = -this._direction._x * 0.4;
                    this._direction._y = -this._direction._y * 0.4;
                    
                    if (!this._showShield) {
                        if (!_view._addHp(-0.07)) {
                            this._died(this);
                        }
                        this._hitTimeout = 30;
                    }
                    return true;
                }
            }
        }

        l = _view._world._layers[_view._enemyLayer];

        for (var i = 0; i < l.length; i++) {
            var o = l[i];
            if (!o._hasDied) {
                if (o._testPoint(p2)) {
                    this._direction._x = -this._direction._x * 0.4;
                    this._direction._y = -this._direction._y * 0.4;

                    if (!this._showShield) {
                        if (!_view._addHp(-0.08)) {
                            this._died(this);
                        }
                        this._hitTimeout = 30;
                    }
                    return true;
                }
            }
        }

        l = _view._world._layers[_view._bonusLayer];
        for (var i = 0; i < l.length; i++) {
            var o = l[i];
            if (!o._isCollected) {
                if (o._testPointInBox(p2)) {
                    o._isCollected = true;
                    _view._bonusCollected(o);
                    return true;
                }
            }
        }

        return false;
    }
}

function Enemy() {
    this._inheritFrom = Player;
    this._inheritFrom();

    this._isHuman = false;
    this._opacity = 0;
    this._speed = 1;
    this._direction = new Point2(0, 0);
    this._hasDied = false;
    this._fireProbability = 0.5;

    this._clone = function () {
        return this._cloneChildrenVectorElementGroup(new Enemy());
    }

    this._render = function (context, camera, offsetX, offsetY) {
        var c = this._children[0];

        offsetX += this._x;
        offsetY += this._y;

        var tx = (offsetX - camera._x) * _screenScale;
        var ty = (offsetY - camera._y) * _screenScale;

        context.save();
        context.globalAlpha = this._opacity;
        context.translate(tx, ty);
        context.translate(((c._x * _screenScale) - c._transformedHalfWidth) >> 0, ((c._y * _screenScale) - c._transformedHalfHeight) >> 0);
        context.translate(c._transformedHalfWidth, c._transformedHalfHeight);
        context.rotate(c._rotate * 0.0174532925199432957);
        context.translate(-c._transformedHalfWidth, -c._transformedHalfHeight);

        context.drawImage(c._cacheCopy, 0, 0);

        context.restore();
    }

    this._update = function () {
        if (this._opacity < 1) {
            this._opacity += 0.05;
        }

        this._children[0]._rotate += 2;

        this._x += this._direction._x * this._speed;
        this._y += this._direction._y * this._speed;

        if (this._x < AREA_MIN_X || this._x > AREA_MAX_X) {
            this._direction._x = -this._direction._x;
        }
        if (this._y < AREA_MIN_Y || this._y > AREA_MAX_Y) {
            this._direction._y = -this._direction._y;
        }

        if (this._hasDied) {
            return false;
        }

        if (!_view._player._hasDied) {
            var d = Common_GetDistanceBetweenPoints(_view._player._x, _view._player._y, this._x, this._y);

            if (d < 400) {
                if (Math.random() < this._fireProbability) {
                    this._doFire();
                }
            }

            this._fireDelay--;
            if (this._fireDelay < 0) {
                this._fireDelay = 0;
            }
        }

        return true;
    }
}

function Bullet() {
    this._inheritFrom = GameUnit;
    this._inheritFrom();

    this._dirX = 0;
    this._dirY = 0;
    this._speed = 0;
    this._owner = null;
    this._gAdj = 1;
    this._strength = -0.1;
    this._applyRotation = true;

    this._clone = function () {
        return this._cloneChildrenVectorElementGroup(new Bullet());
    }

    this._gameUnitDispose = this._dispose;
    this._dispose = function () {
        this._gameUnitDispose();

        this._owner = null;
    }

    this._render = function (context, camera, offsetX, offsetY) {
        var c = this._children[0];

        offsetX += this._x;
        offsetY += this._y;

        var tx = (offsetX - camera._x) * _screenScale;
        var ty = (offsetY - camera._y) * _screenScale;

        context.save();
        context.translate(tx, ty);
        context.translate(((c._x * _screenScale) - c._transformedHalfWidth) >> 0, ((c._y * _screenScale) - c._transformedHalfHeight) >> 0);
        context.translate(c._transformedHalfWidth, c._transformedHalfHeight);
        context.rotate(c._rotate * 0.0174532925199432957);
        context.translate(-c._transformedHalfWidth, -c._transformedHalfHeight);

        context.drawImage(c._cacheCopy, 0, 0);
        context.restore();
    }

    this._move = function (s) {
        if (this._applyRotation) {
            var ox = this._x;
            var oy = this._y;

            this._x += ((this._dirX * this._speed) * s);
            this._y += ((this._dirY * this._speed) * s);

            var r = Common_CalculateAngle(ox, oy, this._x, this._y);
            this._setRotate(r);
        }
        else {
            this._x += ((this._dirX * this._speed) * s);
            this._y += ((this._dirY * this._speed) * s);
        }
    }

    this._update = function () {
        var r = true;

        if (!this._isInPlayingArea()) {
            return false;
        }

        this._move(1);

        if (this._owner._isHuman) {
            if (this._checkHitEnemyUnitHit()) {
                r = false;
            }
        }
        else if (this._indexPart == _view._updateTickSmallPart) {
            if (this._checkHitHumanUnitHit()) {
                r = false;
            }
        }

        return r;
    }

    this._checkHitEnemyUnitHit = function () {
        var l = _view._world._layers[_view._astroidLayer];
        var p = new Point2(this._x, this._y);

        for (var i = 0; i < l.length; i++) {
            var o = l[i];
            if (!o._hasDied) {
                if (o._testPoint(p)) {
                    o._hit(this);

                    if (this._name == "b3") {
                        _view._createSingleExplosion("e1", this._x, this._y, 0);
                    }
                    return true;
                }
            }
        }

        l = _view._world._layers[_view._enemyLayer];

        for (var i = 0; i < l.length; i++) {
            var o = l[i];
            if (!o._hasDied) {
                if (o._testPoint(p)) {
                    o._hit(this);

                    if (this._name == "b3") {
                        _view._createSingleExplosion("e1", this._x, this._y, 0);
                    }
                    return true;
                }
            }
        }

        return false;
    }

    this._checkHitHumanUnitHit = function () {
        var o = _view._player;

        if (!o._hasDied) {
            if (o._testPoint(new Point2(this._x, this._y))) {
                if (o._hitTimeout <= 0 && !o._showShield) {
                    o._hit(this);
                }

                if (this._name == "b3") {
                    _view._createSingleExplosion("e1", this._x, this._y, 0);
                }
                return true;
            }
        }

        return false;
    }

    this._checkHitObjectHit = function () {
        var l = _view._world._layers[_view._objectLayer];

        for (var i = 0; i < l.length; i++) {
            var o = l[i];
            if (o._testPoint(new Point2(this._x, this._y))) {
                o._hit(this);
                _view._objectDestroyed(o);
                return true;
            }
        }

        return false;
    }
}

function BulletLaser() {
    this._inheritFrom = Bullet;
    this._inheritFrom();

    this._speed = 0;
    this._strength = -0.1;
    this._applyRotation = true;
    this._ticksLeft = 90;

    this._clone = function () {
        return this._cloneChildrenVectorElementGroup(new BulletLaser());
    }

    this._render = function (context, camera, offsetX, offsetY) {
        var c = this._children[0];

        offsetX += this._x;
        offsetY += this._y;

        var a = 1;
        if (this._ticksLeft < 10) {
            a = this._ticksLeft * 0.1;
        }
        else if (this._ticksLeft >= 80) {
            a = ((90 - this._ticksLeft) * 0.1);
        }

        var tx = (offsetX - camera._x) * _screenScale;
        var ty = (offsetY - camera._y) * _screenScale;

        context.save();
        context.globalAlpha = a;
        context.translate(tx, ty);
        context.translate(((c._x * _screenScale) - c._transformedHalfWidth) >> 0, ((c._y * _screenScale) - c._transformedHalfHeight) >> 0);
        context.translate(c._transformedHalfWidth, c._transformedHalfHeight);
        context.rotate(c._rotate * 0.0174532925199432957);
        context.translate(-c._transformedHalfWidth, -c._transformedHalfHeight);

        context.drawImage(c._cacheCopy, 0, 0);
        context.restore();
    }

    this._move = function (s) {
        var r = this._owner._children[0]._rotate;
        var p = Common_RotatePoint2(0, -512, MatrixUtility_DegToRad(r), 0, 0);

        this._x = this._owner._x + p._x;
        this._y = this._owner._y + p._y;
        this._setRotate(r);
    }

    this._update = function () {
        var r = true;

        this._move(1);

        if (this._indexPart == _view._updateTickSmallPart) {
            if (this._owner._isHuman) {
                if (this._checkHitEnemyUnitHit()) {
                    r = false;
                }
            }
            else {
            }
        }

        this._ticksLeft--;
        if (this._ticksLeft < 0) {
            return false;
        }

        return r;
    }

    this._checkHitEnemyUnitHit = function () {
        var l = _view._world._layers[_view._astroidLayer];
        var r = this._owner._children[0]._rotate;
        var p = Common_RotatePoint2(0, -1024, MatrixUtility_DegToRad(r), 0, 0);
        var ln = new Array();
        ln.push(new Point2(this._owner._x, this._owner._y));
        ln.push(new Point2(this._owner._x + p._x, this._owner._y + p._y));


        for (var i = 0; i < l.length; i++) {
            var o = l[i];
            if (!o._hasDied) {
                var r = doesLineIntersectPolygonPointList2(o._hitPoints, ln, o._x, o._y);
                if (r != null) {
                    var t = (Math.random() * 5) >> 0;
                    _view._createSingleExplosion("e1", r._x, r._y, t);
                    o._hit(this);
                }
            }
        }

        l = _view._world._layers[_view._enemyLayer];

        for (var i = 0; i < l.length; i++) {
            var o = l[i];
            if (!o._hasDied) {
                var r = doesLineIntersectPolygonPointList2(o._hitPoints, ln, o._x, o._y);
                if (r != null) {
                    var t = (Math.random() * 5) >> 0;
                    _view._createSingleExplosion("e1", r._x, r._y, t);
                    o._hit(this);
                }
            }
        }

        return false;
    }

    this._checkHitHumanUnitHit = function () {
        var o = _view._player;

        if (!o._hasDied) {
            if (o._testPoint(new Point2(this._x, this._y))) {
                o._hit(this);
                //_view._createExp(this);
                return true;
            }
        }

        return false;
    }

    this._checkHitObjectHit = function () {
        var l = _view._world._layers[_view._objectLayer];

        for (var i = 0; i < l.length; i++) {
            var o = l[i];
            if (o._testPoint(new Point2(this._x, this._y))) {
                o._hit(this);
                _view._objectDestroyed(o);
                return true;
            }
        }

        return false;
    }
}

function Explosion() {
    this._inheritFrom = VectorElementGroup;
    this._inheritFrom();

    this._completed = false;
    this._xInc = 0;
    this._yInc = 0;
    this._delay = 0;

    this._clone = function () {
        return this._cloneChildrenVectorElementGroup(new Explosion());
    }

    this._update = function () {
        if (this._delay > 0) {
            this._delay--;
            return true;
        }

        this._isVisible = true;
        this._x += this._xInc;
        this._y += this._yInc;

        return !this._completed;
    }
}

function Bonus() {
    this._inheritFrom = VectorElementGroup;
    this._inheritFrom();

    this._isCollected = false;
    this._index = -1;
    this._spin = 1;
    this._speed = 1;
    this._direction = new Point2(0, 0);
    this._opacity = 0;

    this._clone = function () {
        return this._cloneChildrenVectorElementGroup(new Bonus());
    }

    this._gameUnitUpdate = this._update;
    this._update = function () {
        if (this._isCollected) {
            return false;
        }

        if (this._opacity < 1) {
            this._opacity += 0.05;
        }

        var r = this._children[0]._rotate + this._spin;
        this._setRotate(r);

        if (this._indexPart == _view._updateTickPart) {
            this._updateHitPoints(0);
        }

        this._x += this._direction._x * this._speed;
        this._y += this._direction._y * this._speed;

        if (this._x < AREA_MIN_X || this._x > AREA_MAX_X) {
            this._direction._x = -this._direction._x;
        }
        if (this._y < AREA_MIN_Y || this._y > AREA_MAX_Y) {
            this._direction._y = -this._direction._y;
        }

        return true;
    }

    this._doCollected = function () {
        this._isVisible = false;
        this._isCollected = true;
    }
}