﻿function ModelManager(ctx, img) {
    this._createdPartNames = null;
    this._createdParts = null;
    this._brushCache = null;

    this._backContext = ctx;
    this._imageManager = img;

    this._isInitialized = function () {
        return (this._createdParts != null);
    }

    this._dispose = function () {
        this._cleanModels();

        this._createdPartNames = null;
        this._createdParts = null;
        this._brushCache = null;
        this._backContext = null;
        this._imageManager = null;
    }

    this._cleanModels = function () {
        if (this._createdParts != null) {
            for (var i = 0; i < this._createdPartNames.length; i++) {
                this._createdParts[this._createdPartNames[i]]._dispose();
            }
        }
        if (this._brushCache != null) {
            for (var i = 0; i < this._brushCache.length; i++) {
                this._brushCache[i]._dispose();
            }
        }

        this._createdPartNames = new Array();
        this._createdParts = new Array();
        this._brushCache = new Array();
    }

    this._parseModel = function (x, c) {
        var t = $(x).attr('t');

        if (t == null) {
            t = "VectorElementGroup";
        }
        var o = eval("new " + t + "()");
        this._addModel(o, 0, 0, x, _screenScale, _screenScale);
        o._type = t;
    }

    this._parseBrush = function (id) {
        if (id == "") {
            return null;
        }
        if (this._brushCache[id] != null) {
            return this._brushCache[id];
        }

        var brush = new FaceStyle(null, null, null, null);
        var mgr = this;

        $(_mainXml).find("brushes brush[id='" + id + "']").each(function () {
            var type = $(this).attr('t');

            switch (type) {
                case "image":
                    var _src = $(this).attr('src');
                    brush = new FaceStyle(type, mgr._backContext.createPattern(mgr._imageManager._images[_src], "repeat"), null, null);
                    break;
                case "linear":
                    var x0 = $(this).attr('x0') >> 0;
                    var y0 = $(this).attr('y0') >> 0;
                    var x1 = $(this).attr('x1') >> 0;
                    var y1 = $(this).attr('y1') >> 0;
                    var w = null;
                    var h = null;

                    if ($(this).attr('w') != null) {
                        w = $(this).attr('w') >> 0;
                    }
                    if ($(this).attr('h') != null) {
                        h = $(this).attr('h') >> 0;
                    }

                    var steps = new Array();

                    $(this).find('s').each(function () {
                        var step = new Object();
                        step._start = $(this).attr('p') * 1.0;
                        step._color = $(this).attr('c') + "";
                        steps.push(step);
                    });

                    var desc = new Object();
                    desc._steps = steps;
                    desc._x0 = $(this).attr('x0') * 1;
                    desc._y0 = $(this).attr('y0') * 1;
                    desc._x1 = $(this).attr('x1') * 1;
                    desc._y1 = $(this).attr('y1') * 1;

                    brush = new FaceStyle(type, desc, w, h);
                    break;
                case "radial":
                    var steps = new Array();

                    $(this).find('s').each(function () {
                        var step = new Object();
                        step._start = $(this).attr('p') * 1.0;
                        step._color = $(this).attr('c') + "";
                        steps.push(step);
                    });

                    var desc = new Object();
                    desc._steps = steps;
                    desc._x0 = $(this).attr('x0') * 1;
                    desc._y0 = $(this).attr('y0') * 1;
                    desc._x1 = $(this).attr('x1') * 1;
                    desc._y1 = $(this).attr('y1') * 1;

                    brush = new FaceStyle(type, desc, null, null);
                    break;
                case "solid":
                    brush = new FaceStyle(type, $(this).attr('c'), null, null);
                    break;
                default:
                    brush = new FaceStyle(null, null, null, null);
                    break;
            }
        });

        this._brushCache[id] = brush;

        return brush;
    }

    this._getCanvasUrl = function (id) {
        return this._createdParts[id]._children[0]._cacheCopy.toDataURL();
    }

    this._doesModelExist = function (id) {
        var s = this._createdParts[id];

        return (s != null);
    }

    this._addModelFromCache = function (id, x, y) {
        if(this._doesModelExist(id)) {
            var s = this._createdParts[id];
            var c = s._clone();
            c._x = x;
            c._y = y;
            return c;
        }
        else {
            console.log("Model not found: " + id);
        }
    }

    this._addModel = function (model, x, y, xml, scaleX, scaleY) {
        var mXml = xml;
        var mgr = this;
        var mid = $(xml).attr('id');

        if (model == null) {
            model = new VectorElementGroup();
        }

        model._name = mid;
        model._x = x;
        model._y = y;
        model._minWidth = 9999;
        model._maxWidth = -9999;
        model._minHeight = 9999;
        model._maxHeight = -9999;

        mXml.find('frames f').each(function () {
            var a = new VectorElementAnimation();

            a._id = $(this).attr('id');
            a._delay = $(this).attr('d') >> 0;
            a._ticksTillNextFrame = a._delay;
            a._doLoop = ($(this).attr('l') == "true");
            a._autoRun = ($(this).attr('ar') == "true");

            sp = $(this).attr('s').split(',');
            for (var i = 0; i < sp.length; i++) {
                var x = sp[i].split('-');
                var y = new Array();
                for (var ii = 0; ii < x.length; ii++) {
                    y.push(x[ii] >> 0);
                }

                a._sequence.push(y);
            }

            var r = $(this).attr('r');
            if (r != null && r != "") {
                sp = r.split(',');
                for (var i = 0; i < sp.length; i++) {
                    a._required.push(sp[i] >> 0);
                }
            }

            model._addAnimation(a);
        });

        mXml.find('p').each(function () {
            var part = $(this);
            var partX = 0;
            var partY = 0;
            var partRotate = 0;
            var minX = 9999;
            var maxX = -9999;
            var minY = 9999;
            var maxY = -9999;
            var polys = new Array();
            var cache = part.attr('cache') != "false";
            var render = part.attr('render');
            var isVisible = (part.attr('visible') + '' != "false");
            var ptype = (part.attr('t'));
            var us = false;
            var sw = 0;
            var sh = 0;

            if (part.attr('x') != null) {
                partX = part.attr('x') * 1;
            }
            if (part.attr('y') != null) {
                partY = part.attr('y') * 1;
            }

            if (part.attr('w') != null) {
                sw = part.attr('w') >> 0;
                us = true;
            }
            if (part.attr('h') != null) {
                sh = part.attr('h') >> 0;
                us = true;
            }

            if (part.attr('r') != null) {
                partRotate = part.attr('r') * 1;
            }

            if (render == null) {
                render = 1;
            }
            else {
                render >>= 0;
            }

            var cl = new Array();
            var fc = 0;
            var ib = false;

            if (part.attr('s') != null) {
                var poly = new BitmapShape();

                poly._src = part.attr('s') + '';
                if (part.attr('c') == '1') {
                    poly._hitTestMode = 2;
                }

                var img = _imageManager._images[poly._src];
                sw = img.width;
                sh = img.height;

                poly._scaleX = 0.5;
                if (part.attr('sx') != null) {
                    poly._scaleX = part.attr('sx') * 1;
                }
                poly._scaleY = 0.5;
                if (part.attr('sy') != null) {
                    poly._scaleY = part.attr('sy') * 1;
                }

                sw *= poly._scaleX;
                sh *= poly._scaleY;

                us = true;

                poly._width = sw;
                poly._height = sh;
                poly._halfWidth = sw * 0.5;
                poly._halfHeight = sh * 0.5;
                poly._rotate = 0;
                poly._x = poly._width * 0.5;
                poly._y = poly._height * 0.5;

                polys.push(poly);

                if (poly._x < minX) {
                    minX = poly._x;
                }
                if (poly._y < minY) {
                    minY = poly._y;
                }
                if (poly._x + poly._width > maxX) {
                    maxX = poly._x + poly._width;
                }
                if (poly._y + poly._height > maxY) {
                    maxY = poly._y + poly._height;
                }

                if (poly._hitTestMode > 0) {
                    cl.push(fc);
                }
                ib = true;
                fc++;
            }

            part.find('f').each(function () {
                var type = $(this).attr('type') + '';
                var _offset = new Vector2(0, 0);

                if ($(this).attr('ox') != null) {
                    _offset._x = $(this).attr('ox') >> 0;
                }
                if ($(this).attr('oy') != null) {
                    _offset._y = $(this).attr('oy') >> 0;
                }
                var _shadowColor = $(this).attr('sc');
                var _shadowBlur = $(this).attr('sb') * 1;
                var _shadowOffsetX = $(this).attr('sox') * 1;
                var _shadowOffsetY = $(this).attr('soy') * 1;

                switch (type) {
                    case "effect":
                        var poly = new EffectShape();
                        poly._effectJs = $(this).text();
                        polys.push(poly);
                        break;
                    case "text":
                        var poly = new TextBlock();
                        var points = $(this).attr('p').split(',');
                        var t = $(this).attr('st') * 1;
                        var f = $(this).attr('b') + '';
                        var s = $(this).attr('s') + '';
                        var w = $(this).attr('w') * 1;
                        var h = $(this).attr('h') * 1;
                        var ha = $(this).attr('ha') >> 0;
                        var va = $(this).attr('va') >> 0;
                        poly._x = points[0] * 1;
                        poly._y = points[1] * 1;
                        poly._fontFamily = $(this).attr('ff') + '';
                        poly._fontSize = $(this).attr('fs') + '';
                        poly._text = $(this).attr('v') + '';

                        poly._width = w;
                        poly._height = h;
                        poly._halfWidth = poly._width * 0.5;
                        poly._halfHeight = poly._height * 0.5;
                        poly._longDim = w;
                        poly._hAlign = ha;
                        poly._vAlign = va;

                        poly._shadowColor = _shadowColor;
                        poly._shadowBlur = _shadowBlur;
                        poly._shadowOffsetX = _shadowOffsetX;
                        poly._shadowOffsetY = _shadowOffsetY;

                        poly._setStyles(mgr._parseBrush(f), mgr._parseBrush(s), t);
                        polys.push(poly);

                        if (poly._x - poly._halfWidth < minX) {
                            minX = poly._x - poly._halfWidth;
                        }
                        if (poly._x + poly._halfWidth > maxX) {
                            maxX = poly._x + poly._halfWidth;
                        }
                        if (poly._y - poly._halfHeight < minY) {
                            minY = poly._y - poly._halfHeight;
                        }
                        if (poly._y + poly._halfHeight > maxY) {
                            maxY = poly._y + poly._halfHeight;
                        }
                        break;
                    case "poly":
                        var poly = new Polygon();
                        var points = $(this).attr('p').split(',');
                        var t = $(this).attr('st') * 1;
                        var f = $(this).attr('b') + '';
                        var s = $(this).attr('s') + '';
                        var sx = $(this).attr('sx') * 1;
                        var sy = $(this).attr('sy') * 1;
                        poly._isCollidable = ($(this).attr('col') + '' == "true");
                        poly._isVisible = ($(this).attr('visible') + '' != "false");

                        var hm = $(this).attr('colmode');
                        if (hm == null) {
                            hm = "";
                        }
                        if ($(this).attr('sx') == null) {
                            sx = 1;
                        }
                        if ($(this).attr('sy') == null) {
                            sy = 1;
                        }
                        poly._hitTestMode = (hm != "" ? parseInt(hm) : 0);
                        poly._minX = 9999;
                        poly._maxX = -9999;
                        poly._minY = 9999;
                        poly._maxY = -9999;

                        poly._shadowColor = _shadowColor;
                        poly._shadowBlur = _shadowBlur;
                        poly._shadowOffsetX = _shadowOffsetX;
                        poly._shadowOffsetY = _shadowOffsetY;

                        for (i = 0; i < points.length; i += 2) {
                            var v = new Vector2((_offset._x + (points[i] * 1)) * sx, (_offset._y + (points[i + 1] * 1)) * sy);
                            poly._points.push(v);

                            if (v._x < minX) {
                                minX = v._x;
                            }
                            if (v._y < minY) {
                                minY = v._y;
                            }
                            if (v._x > maxX) {
                                maxX = v._x;
                            }
                            if (v._y > maxY) {
                                maxY = v._y;
                            }

                            if (v._x < poly._minX) {
                                poly._minX = v._x;
                            }
                            if (v._y < poly._minY) {
                                poly._minY = v._y;
                            }
                            if (v._x > poly._maxX) {
                                poly._maxX = v._x;
                            }
                            if (v._y > poly._maxY) {
                                poly._maxY = v._y;
                            }
                        }

                        poly._width = Math.abs(poly._maxX - poly._minX);
                        poly._height = Math.abs(poly._maxY - poly._minY);
                        poly._halfWidth = poly._width * 0.5;
                        poly._halfHeight = poly._height * 0.5;
                        poly._longDim = poly._width;
                        if (poly._height > poly._longDim) {
                            poly._longDim = poly._height;
                        }

                        if ($(this).attr('console') == 'true') {
                            var s = "";
                            for (var i = 0; i < poly._points.length; i++) {
                                if (s != "") {
                                    s += ",";
                                }
                                s += poly._points[i]._x + "," + poly._points[i]._y;
                            }
                        }

                        poly._setStyles(mgr._parseBrush(f), mgr._parseBrush(s), t);

                        poly._points.push(new Vector2((_offset._x + (points[0] * 1)) * sx, (_offset._y + (points[1] * 1)) * sy));

                        if ($(this).find('i').length > 0) {
                            $(this).find('i').each(function () {
                                var ix = $(this).attr('x') * 1;
                                var iy = $(this).attr('y') * 1;
                                var fc = poly._clone();
                                for (i = 0; i < fc._points.length; i++) {
                                    fc._points[i]._x += ix;
                                    fc._points[i]._y += iy;
                                }
                                polys.push(fc);
                            });
                        }
                        else {
                            polys.push(poly);
                        }
                        break;
                    case "circle":
                        var poly = new CircleShape();
                        var points = $(this).attr('p').split(',');
                        var t = $(this).attr('st') * 1;
                        var f = $(this).attr('b') + '';
                        var s = $(this).attr('s') + '';
                        var r = $(this).attr('r') * 1;
                        var cx = points[0] * 1;
                        var cy = points[1] * 1;
                        polys.push(mgr._createCircle(cx, cy, r, f, s, t, ($(this).attr('col') + '' == "true")));

                        if (cx - r < minX) {
                            minX = cx - r;
                        }
                        if (cy - r < minY) {
                            minY = cy - r;
                        }
                        if (cx + r > maxX) {
                            maxX = cx + r;
                        }
                        if (cy + r > maxY) {
                            maxY = cy + r;
                        }
                        break;
                    case "bmp":
                        var poly = new BitmapShape();
                        var points = $(this).attr('p').split(',');

                        poly._src = $(this).attr('s') + '';
                        poly._x = points[0] * 1;
                        poly._y = points[1] * 1;
                        poly._width = $(this).attr('w') * 1;
                        poly._height = $(this).attr('h') * 1;
                        poly._scaleX = $(this).attr('sx') * 1;
                        poly._scaleY = $(this).attr('sy') * 1;

                        if ($(this).attr('r') != null) {
                            poly._rotate = $(this).attr('r') * 1;
                        }

                        polys.push(poly);

                        if (poly._x < minX) {
                            minX = poly._x;
                        }
                        if (poly._y < minY) {
                            minY = poly._y;
                        }
                        if (poly._x + poly._width > maxX) {
                            maxX = poly._x + poly._width;
                        }
                        if (poly._y + poly._height > maxY) {
                            maxY = poly._y + poly._height;
                        }
                        break;
                }
                
                if (poly._hitTestMode > 0) {
                    cl.push(fc);
                }
                fc++;
            });

            var longW = 0;
            var longH = 0;
            var ds = 2;

            if (us) {
                model._minWidth = -(sw * 0.5);
                model._maxWidth = (sw * 0.5);
                model._minHeight = -(sh * 0.5);
                model._maxHeight = (sh * 0.5);

                if (mid == "pb") {
                    console.log(mid + ":" + model._minWidth + "," + model._maxWidth);
                }

                longW = (sw >> 1);
                longH = (sh >> 1);
                if (mid == "b1" || mid == "b2" || mid == "b3" || mid == "b4" || mid == "b5" || mid == "b6" || mid == "b7" || mid == "b8" || mid == "b9" || mid == "b9" || mid == "b10") {
                    ds = 1;
                }
                else {
                    ds = 0;
                }
            }
            else {
                if (partX + minX < model._minWidth) {
                    model._minWidth = partX + minX;
                }
                if (partY + minY < model._minHeight) {
                    model._minHeight = partY + minY;
                }
                if (partX + maxX > model._maxWidth) {
                    model._maxWidth = partX + maxX;
                }
                if (partY + maxY > model._maxHeight) {
                    model._maxHeight = partY + maxY;
                }

                longW = Math.abs(minX);
                if (Math.abs(maxX) > longW) {
                    longW = Math.abs(maxX);
                }
                longH = Math.abs(minY);
                if (Math.abs(maxY) > longH) {
                    longH = Math.abs(maxY);
                }
            }

            var w = ((longW * 2) + ds) >> 0;
            var h = ((longH * 2) + ds) >> 0;

            if (ib) {
                w = polys[0]._width;
                h = polys[0]._height;
            }

            if (mid == "pb") {
            }
            var modelPart = null;

            if (ptype != null) {
                modelPart = eval("new " + ptype + "(" + w + "," + h + ", null, null)");
            }

            if (modelPart == null) {
                modelPart = new SingleVectorElement(w, h, null, null);
            }

            modelPart._enableCaching = cache;
            modelPart._parent = model;
            modelPart._x = partX;
            modelPart._y = partY;
            modelPart._setRotate(partRotate);

            for (var i = 0; i < polys.length; i++) {
                modelPart._faces.push(polys[i]);
            }

            if (cache) {
                modelPart._setSourceTransforms(scaleX, scaleY);
            }
            else {
                modelPart._setSourceTransformsNoRebuild(scaleX, scaleY);
            }

            modelPart._setScale(1, 1);
            modelPart._hitTestFaceList = cl;

            switch (render) {
                case 1:
                    modelPart._render = modelPart._renderMode1;
                    break;
                case 2:
                    modelPart._render = modelPart._renderMode2;
                    break;
            }

            model._children.push(modelPart);
        });

        var mx = Math.abs(model._minWidth) + Math.abs(model._maxWidth);
        var my = Math.abs(model._minHeight) + Math.abs(model._maxHeight);

        model._bidDim = mx;
        if (mx < my) {
            model._bidDim = my;
        }

        model._minWidth = 9999;
        model._maxWidth = -9999;
        model._minHeight = 9999;
        model._maxHeight = -9999;

        this._createdParts[mid] = model;
        this._createdPartNames.push(mid);

        return model;
    }

    this._createCircle = function (x, y, r, f, s, t, ic) {
        var poly = new CircleShape();
        poly._isCollidable = ic;
        poly._x = x;
        poly._y = y;
        poly._radius = r;

        poly._width = r * 2;
        poly._height = r * 2;
        poly._halfWidth = r * 0.5;
        poly._halfHeight = r * 0.5;
        poly._longDim = r * 2;

        poly._setStyles(this._parseBrush(f), this._parseBrush(s), t);
        return poly;
    }

    this._createPoly = function (p, f, s, t, ic, iv) {
        var poly = new Polygon();
        var points = p.split(',');
        var sx = 1;
        var sy = 1;
        poly._isCollidable = ic;
        poly._isVisible = iv;
        poly._hitTestMode = 1;
        poly._minX = 9999;
        poly._maxX = -9999;
        poly._minY = 9999;
        poly._maxY = -9999;
        poly._shadowColor = null;

        for (i = 0; i < points.length; i += 2) {
            var v = new Vector2(points[i] * 1, points[i + 1] * 1);
            poly._points.push(v);

            if (v._x < poly._minX) {
                poly._minX = v._x;
            }
            if (v._y < poly._minY) {
                poly._minY = v._y;
            }
            if (v._x > poly._maxX) {
                poly._maxX = v._x;
            }
            if (v._y > poly._maxY) {
                poly._maxY = v._y;
            }
        }

        poly._width = Math.abs(poly._maxX - poly._minX);
        poly._height = Math.abs(poly._maxY - poly._minY);
        poly._halfWidth = poly._width * 0.5;
        poly._halfHeight = poly._height * 0.5;
        poly._longDim = poly._width;
        if (poly._height > poly._longDim) {
            poly._longDim = poly._height;
        }

        poly._setStyles(this._parseBrush(f), this._parseBrush(s), t);
        poly._points.push(new Vector2(points[0] * 1, points[1] * 1));

        return poly;
    }

    this._applyScale = function (sx, sy) {
        for (var a = 0; a < this._createdPartNames.length; a++) {
            var o = this._createdParts[this._createdPartNames[a]];

            for (var b = 0; b < o._children.length; b++) {
                var p = o._children[b];
                p._setSourceTransforms(sx, sy);
            }
        }
    }

    this._cleanModels();
}
