﻿function BoardBar() {
    this._inheritFrom = VectorElementGroup;
    this._inheritFrom();

    this._tickValue = -0.00025;
    this._value = 0;
    this._target = 0;

    this._clone = function () {
        return this._cloneChildrenVectorElementGroup(new BoardBar());
    }

    this._render = function (ctx, c, ox, oy) {
        ctx.save();

        var ch = this._children[0];
        var tx = ((ch._x + (this._x - c._x)) * _screenScale) - ch._transformedHalfWidth;
        var ty = ((ch._y + (this._y - c._y)) * _screenScale) - ch._transformedHalfHeight;

        ctx.drawImage(ch._cacheCopy, (0.5 + tx) >> 0, (0.5 + ty) >> 0);

        if (this._value > 0) {
            var ch = this._children[1];
            var tx = ((ch._x + (this._x - c._x)) * _screenScale) - ch._transformedHalfWidth;
            var ty = ((ch._y + (this._y - c._y)) * _screenScale) - ch._transformedHalfHeight;

            var w = (this._value * ch._transformedWidth);

            if (w >= 1) {
                ctx.beginPath();
                ctx.rect(tx, ty, w >> 0, ch._transformedHeight >> 0);
                ctx.clip();

                ctx.drawImage(ch._cacheCopy, (0.5 + tx) >> 0, (0.5 + ty) >> 0);
            }
        }

        ctx.restore();
    }

    this._next = function () {
        return this._add(this._tickValue);
    }

    this._add = function (v) {
        this._target += v;
        if (this._target <= 0) {
            this._target = 0;
            return false;
        }
        else if (this._target > 1) {
            this._target = 1;
        }

        return true;
    }

    this._update = function () {
        var d = (this._target - this._value) * 0.2;
        this._value += d;

        return true;
    }
}

function BoardButton2() {
    this._inheritFrom = BoardButton;
    this._inheritFrom();

    this._clone = function () {
        return this._cloneChildrenVectorElementGroup(new BoardButton2());
    }

    this._reset = function () {
        this._children[0]._opacity = 1;
        this._isPressed = false;
        this._releaseTicks = 0;
    }

    this._inputDown = function () {
        this._children[0]._opacity = 0.7;
        this._isPressed = true;
        this._releaseTicks = 40;
    }

    this._disable = function () {
        this._children[0]._opacity = 0.3;
        this._isDisabled = true;
    }
}

function BoardButton() {
    this._inheritFrom = VectorElementGroup;
    this._inheritFrom();

    this._isButton = true;
    this._isPressed = false;
    this._isDisabled = false;

    this._releaseTicks = 0;

    this._clone = function () {
        return this._cloneChildrenVectorElementGroup(new BoardButton());
    }

    this._reset = function () {
        this._play("n");
        this._isPressed = false;
        this._releaseTicks = 0;
    }

    this._inputDown = function () {
        if (!this._isPressed) {
            this._play("c");
            this._isPressed = true;
            this._releaseTicks = 40;
        }
    }

    this._disable = function () {
        this._play("d");
        this._isDisabled = true;
    }

    this._hitTest = function (c, x, y) {
        var a;
        var hw = this._children[0]._width >> 1;
        var hh = this._children[0]._height >> 1;

        if (x >= this._x - hw && x < this._x + hw && y >= this._y - hh && y < this._y + hh) {
            return true;
        }

        return false;
    }

    this._update = function () {
        if (this._releaseTicks > 0) {
            this._releaseTicks--;
            if (this._releaseTicks == 0) {
                this._isPressed = false;
            }
        }

        return true;
    }
}

function BoardText() {
    this._inheritFrom = VectorElementGroup;
    this._inheritFrom();

    this._text = "";
    this._charIndices = null;

    this._clone = function () {
        return this._cloneChildrenVectorElementGroup(new BoardText());
    }

    this._setText = function (s) {
        this._text = s.toUpperCase();

        this._charIndices = new Array();
        for (var i = 0; i < this._text.length; i++) {
            var c = this._text.charCodeAt(i);
            if (c >= 48) {
                if (c >= 65) {
                    c -= 7;
                }

                this._charIndices.push(c - 48);
            }
            else {
                if (c == 32) {
                    c = 150;
                    this._charIndices.push(c);
                }
            }
        }
    }

    this._render = function (ctx, c, ox, oy) {
        if (this._text != "") {
            var ch = this._children[0];

            ctx.save();

            var tx = ((ch._x + ((this._x + ox) - c._x)) * _screenScale) - ch._transformedHalfWidth;
            var ty = ((ch._y + ((this._y + oy) - c._y)) * _screenScale) - ch._transformedHalfHeight;
            var w = ch._transformedWidth;

            for (var i = 0; i < this._charIndices.length; i++) {
                var c = this._charIndices[i];

                if (c == 150) {
                    tx += (w >> 1) - 1;
                }
                else {
                    ch = this._children[c];
                    w = ch._transformedWidth;
                    ctx.drawImage(ch._cacheCopy, (0.5 + tx) >> 0, (0.5 + ty) >> 0);
                    tx += w - 1;
                }
            }

            ctx.restore();
        }
    }

    this._formatNumberAsText = function (s, c) {
        this._setText(padx(s, c));
    }

    this._update = function () {
        return true;
    }
}